# -*- coding: utf-8 -*-

from six import with_metaclass

from .query import QueryObject


def page_object_factory(page_element):
    def wrapper(self):
        we = None

        for query_object in page_element.query_set[:-1]:
            if we:
                we = we.query.from_object(
                    query_object,
                ).first()
            else:
                we = self.query.from_object(
                    query_object,
                ).first()

        if we:
            result = we.query.from_object(page_element.query_set[-1:][0])
        else:
            result = self.query.from_object(page_element.query_set[-1:][0])

        if page_element.wait_timeout:
            result.wait(
                page_element.wait_timeout,
            )

        if page_element.is_list:
            result = result.all()
        elif page_element.index:
            result = result.get(
                page_element.index,
            )
        else:
            result = result.first()

        if page_element.proxy_class:
            return page_element.proxy_class(result)
        return result

    return property(wrapper)


class PageObjectProxy(object):

    def __init__(self, obj):
        self.__obj = obj

    def __getattr__(self, item):
        return getattr(self.__obj, item)

    def __repr__(self):
        return repr(self.result)

    @property
    def obj(self):
        return self.__obj


class PageObject(object):

    def __init__(self, *query_set, **options):
        assert bool(query_set), 'Element can not be created without query set'

        if isinstance(query_set[0], PageObject):
            self.__query_set = query_set[0].query_set + query_set[1:]
        else:
            self.__query_set = query_set

        self.__index = options.get('index', None)
        self.__is_list = options.get('is_list', False)
        self.__proxy_class = options.get('proxy_class', None)
        self.__wait_timeout = options.get('wait_timeout', None)

    @property
    def query_set(self):
        return self.__query_set

    @property
    def index(self):
        return self.__index

    @property
    def is_list(self):
        return self.__is_list

    @property
    def proxy_class(self):
        return self.__proxy_class

    @proxy_class.setter
    def proxy_class(self, cls):
        assert issubclass(cls, PageObjectProxy), \
            '"{}" is not "PageObjectProxy" subclass'.format(
                cls.__name__,
            )
        self.__proxy_class = cls

    @property
    def wait_timeout(self):
        return self.__wait_timeout


class PageApi(object):

    def __init__(self, page):
        self.__page = page

    @property
    def page(self):
        return self.__page


class PageMeta(type):
    """
    Factory for creating page object class
    """

    def __new__(mcs, name, bases, dct):
        cls = type.__new__(mcs, name, bases, dct)

        cls_items = (
            (a, getattr(cls, a, None))
            for a in dir(cls)
            if not a.startswith('_')
        )

        for atr, value in cls_items:
            if isinstance(value, PageObject):
                setattr(cls, atr, page_object_factory(value))

        return cls


class Forms(object):

    def __init__(self, **classes):
        self.__area = None
        self.__instances = {}

        self.__classes = classes

    def __getattr__(self, item):
        if not self.__area:
            raise RuntimeError('Need area for create instance')

        if item not in self.__instances:
            self.__instances[item] = self.__classes[item](self.__area)

        return self.__instances[item]

    def __call__(self, proxy):
        inst = self.__class__(**self.__classes)
        inst.change_area(proxy)
        return inst

    def change_area(self, proxy):
        self.__area = proxy

    def refresh(self):
        self.__instances = {}


class Page(with_metaclass(PageMeta, object)):

    __wrapper__ = None
    __forms__ = Forms()
    __api_class__ = PageApi

    def __init__(self, proxy):
        self.__proxy = proxy
        self.__api = self.__api_class__(self)
        self.__forms = self.__forms__(self.area)

    @property
    def api(self):
        return self.__api

    @property
    def area(self):
        if self.__wrapper__:
            if not isinstance(self.__wrapper__, QueryObject):
                raise TypeError(
                    '"__wrapper__" can be instance of QueryObject only',
                )
            return self.__proxy.query.from_object(
                self.__wrapper__,
            ).first()

        return self.__proxy

    @property
    def driver(self):
        return self.__proxy.driver

    @property
    def query(self):
        return self.area.query

    @property
    def forms(self):
        return self.__forms

    def relate_to(self, proxy):
        self.__proxy = proxy
        self.__forms.change_area(self.area)

    def refresh(self, force=False):
        if force:
            self.__proxy.driver.refresh()
        self.__forms.refresh()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from multiprocessing.pool import ThreadPool

from .. import runnable
from ..tools import get_pool_size_of_value
from ..exceptions import ALLOW_RAISED_EXCEPTIONS


def target(runnable_object, result):
    runnable_object(result)


class ThreadingSuiteGroup(runnable.RunnableGroup):

    def run(self, result):
        self._is_run = True

        pool = ThreadPool(
            get_pool_size_of_value(self.config.ASYNC_SUITES),
        )

        try:
            for suite in self.objects:
                pool.apply_async(target, args=(suite, result))

            pool.close()
            pool.join()
        except ALLOW_RAISED_EXCEPTIONS:
            pool.terminate()
            raise


class ThreadingCaseGroup(runnable.RunnableGroup):

    def run(self, result):
        self._is_run = True

        pool = ThreadPool(
            get_pool_size_of_value(self.config.ASYNC_TESTS, in_two=True),
        )

        try:
            for case in self.objects:
                pool.apply_async(target, args=(case, result))

            pool.close()
            pool.join()
        except ALLOW_RAISED_EXCEPTIONS:
            pool.terminate()
            raise



# -*- coding: utf-8 -*-

import re
import sys
import traceback

from . import config
from . import loader
from . import runnable
from .utils import pyv
from . import collector
from . import extensions
from .suite import Suite
from .result import Result
from .ext import EXTENSIONS
from .utils.common import measure_time
from .utils.common import call_to_chain
from .groups.default import DefaultSuiteGroup
from .exceptions import ALLOW_RAISED_EXCEPTIONS


DEFAULT_LAYERS = []
CONFIG_ENV_NAME = 'SEISMOGRAPH_CONF'


class ProgramLayer(runnable.LayerOfRunnableObject):

    def on_init(self, program):
        """
        :type program: Program
        """
        pass

    def on_config(self, config):
        """
        :type config: seismograph.config.Config
        """
        pass

    def on_setup(self, program):
        """
        :type program: Program
        """
        pass

    def on_teardown(self, program):
        """
        :type program: Program
        """
        pass

    def on_error(self, error, program, result):
        """
        :type error: BaseException
        :type program: Program
        :type result: seismograph.result.Result
        """
        pass

    def on_option_parser(self, parser):
        """
        :param parser: optparse.OptionParser
        """
        pass

    def on_run(self, program):
        """
        :type program: Program
        """
        pass


class ProgramContext(runnable.ContextOfRunnableObject):

    def __init__(self, setup, teardown):
        self.__layers = []

        self.__setup_callbacks = [setup]
        self.__teardown_callbacks = [teardown]

    @property
    def setup_callbacks(self):
        return self.__setup_callbacks

    @property
    def teardown_callbacks(self):
        return self.__teardown_callbacks

    @property
    def layers(self):
        for layer in DEFAULT_LAYERS:
            if layer.enabled:
                yield layer

        for layer in self.__layers:
            if layer.enabled:
                yield layer

    def add_layers(self, layers):
        self.__layers.extend(layers)

    def start_context(self, program):
        try:
            call_to_chain(self.layers, 'on_setup', program)
            call_to_chain(self.__setup_callbacks, None)
        except BaseException:
            runnable.stopped_on(program, 'start_context')
            raise

    def stop_context(self, program):
        try:
            call_to_chain(self.layers, 'on_teardown', program)
            call_to_chain(self.__teardown_callbacks, None)
        except BaseException:
            runnable.stopped_on(program, 'stop_context')
            raise

    def on_init(self, program):
        call_to_chain(self.layers, 'on_init', program)

    def on_config(self, program, config):
        call_to_chain(self.layers, 'on_config', config)

    def on_error(self, error, program, result):
        try:
            call_to_chain(self.layers, 'on_error', error, program, result)
        except BaseException:
            runnable.stopped_on(program, 'on_error')
            raise

    def on_option_parser(self, parser):
        call_to_chain(self.layers, 'on_option_parser', parser)

    def on_run(self, program):
        try:
            call_to_chain(self.layers, 'on_run', program)
        except BaseException:
            runnable.stopped_on(program, 'on_run')
            raise


class Program(runnable.RunnableObject):

    __layers__ = None
    __suite_class__ = Suite
    __create_reason__ = False
    __result_class__ = Result
    __suite_group_class__ = None
    __config_class__ = config.Config

    #
    # Base components of runnable object
    #

    def __is_run__(self):
        return self.__is_run

    #
    # Self code is starting here
    #

    def __init__(self,
                 suites_path='__main__',
                 recursive_load=True,
                 scripts=None,
                 config_path=None,
                 layers=None,
                 argv=None,
                 exit=True,
                 suites=None):
        super(Program, self).__init__()

        if argv:
            sys.argv.extend(argv)

        self.suites_path = suites_path
        self.recursive_load = recursive_load

        self.__suites = []
        self.__exit = exit
        self.__is_run = False

        self.__context = ProgramContext(self.setup, self.teardown)

        if layers:
            self.__context.add_layers(layers)

        if self.__layers__:
            self.__context.add_layers(self.__layers__)

        parser = config.create_option_parser()
        self.__context.on_option_parser(parser)
        for ext in EXTENSIONS:
            extensions.add_options(ext, parser)
        options, _ = parser.parse_args()

        self.__config = self.__config_class__(
            config.get_config_path_by_env(
                CONFIG_ENV_NAME, default=config_path,
            ),
            options=options,
        )

        config.prepare_config(self.__config)
        self.__context.on_config(self, config)

        if self.__config.NO_SKIP:
            from .case import set_no_skip
            set_no_skip()

        if self.__config.NO_SCRIPTS:
            self.__scripts = []
        else:
            self.__scripts = [
                s(self) for s in (scripts or [])
            ] if scripts else []

        for ext in EXTENSIONS:
            extensions.install(ext(), self)

        if suites:
            self.register_suites(suites)

        self.__context.on_init(self)

    @property
    def exit(self):
        return self.__exit

    @exit.setter
    def exit(self, value):
        self.__exit = value

    @property
    def suites(self):
        return self.__suites

    @property
    def config(self):
        return self.__config

    @property
    def context(self):
        return self.__context

    @property
    def scripts(self):
        return self.__scripts

    def _make_group(self):
        if self.__suite_group_class__:
            return self.__suite_group_class__(
                self.__suites, self.__config,
            )

        if self.config.GEVENT:
            pyv.check_gevent_supported()

            from .groups.gevent import GeventSuiteGroup

            return GeventSuiteGroup(
                self.__suites, self.__config,
            )

        if self.config.THREADING:
            from .groups.threading import ThreadingSuiteGroup

            return ThreadingSuiteGroup(
                self.__suites, self.__config,
            )

        if self.config.MULTIPROCESSING:
            from .groups.multiprocessing import MultiprocessingSuiteGroup

            return MultiprocessingSuiteGroup(
                self.__suites, self.__config,
            )

        return DefaultSuiteGroup(
            self.__suites, self.__config,
        )

    def _make_result(self):
        if self.__config.OUTPUT:
            stdout = open(self.__config.OUTPUT, 'w')
        else:
            stdout = sys.stdout

        return self.__result_class__(
            self.__config,
            stdout=stdout,
        )

    @staticmethod
    def ext(name):
        return extensions.get(name)

    def setup(self, *args, **kwargs):
        pass

    def teardown(self, *args, **kwargs):
        pass

    def add_setup(self, f):
        self.__context.setup_callbacks.append(f)
        return f

    def add_teardown(self, f):
        self.__context.teardown_callbacks.append(f)
        return f

    @staticmethod
    def shared_data(name, data):
        extensions.set(data, name, is_data=True)

    @staticmethod
    def shared_extension(name, ext, singleton=False, args=None, kwargs=None):
        extensions.set(
            ext,
            name,
            is_data=False,
            singleton=singleton,
            args=args, kwargs=kwargs,
        )

    def suite_is_valid(self, suite):
        is_valid = True

        if self.__config.INCLUDE_SUITES_PATTERN:
            is_valid = bool(
                re.search(self.__config.INCLUDE_SUITES_PATTERN, suite.name),
            )

        if self.__config.EXCLUDE_SUITE_PATTERN:
            is_valid = not bool(
                re.search(self.__config.EXCLUDE_SUITE_PATTERN, suite.name),
            )

        return is_valid

    def register_suite(self, suite):
        if self.suite_is_valid(suite):
            suite.mount_to(self)

    def register_suites(self, suites):
        for suite in suites:
            self.register_suite(suite)

    def load_suites(self, path=None):
        path = path or self.suites_path

        if path:
            if path == '__main__':
                self.register_suites(
                    loader.load_suites_from_module(
                        __import__('__main__'),
                        self.__suite_class__,
                    ),
                )
            else:
                if path not in sys.path:
                    sys.path.append(path)

                self.register_suites(
                    loader.load_suites_from_path(
                        path,
                        self.__suite_class__,
                        recursive=self.recursive_load,
                    ),
                )

    def run_scripts(self, result, run_point=None):
        if run_point:
            scripts = filter(
                lambda s: s.__run_point__ == run_point, self.__scripts,
            )
        else:
            scripts = self.__scripts

        for script in scripts:
            with result.proxy(script) as result_proxy:
                script(result_proxy)
            if result.current_state.should_stop:
                break

    def run(self):
        self.__is_run = True

        result = self._make_result()

        if self.suites_path:
            self.load_suites()

        if not self.__suites and not self.__scripts:
            raise RuntimeError('No suites or scripts for execution')

        self.__suites = collector.create_generator(
            self.__suites, self.__config,
        )

        if self.__config.TREE:
            from .tree import print_tree
            print_tree(self.__suites)

        group = self._make_group()
        timer = measure_time()

        with result:
            try:
                self.__context.on_run(self)

                with self.__context(self):
                    self.run_scripts(result, run_point='before')
                    group(result)
                    self.run_scripts(result, run_point='after')
            except ALLOW_RAISED_EXCEPTIONS:
                raise
            except BaseException as error:
                result.add_error(
                    self, traceback.format_exc(), timer(), error,
                )
                self.__context.on_error(error, self, result)

        if self.__exit:
            sys.exit(not result.current_state.was_success)

        return result.current_state.was_success


def main(*args, **kwargs):
    Program(*args, **kwargs).run()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import logging

from selenium.common.exceptions import WebDriverException
from selenium.common.exceptions import NoSuchElementException

from ...utils.common import waiting_for
from ...exceptions import TimeoutException
from .utils import change_name_from_python_to_html


logger = logging.getLogger(__name__)


TAG_ALIASES = {
    'link': 'a',
}
ATTRIBUTE_ALIASES = {
    '_id': 'id',
    '_class': 'class',
    '_type': 'type',
}


class QueryObject(object):

    def __init__(self, tag, **selector):
        self.tag = tag
        self.selector = selector

    def __str__(self):
        return self.__unicode__()

    def __unicode__(self):
        return u'<{} {}>'.format(
            self.tag,
            u' '.join(
                (u'{}="{}"'.format(k, v) for k, v in self.selector.items())
            ),
        )


class _Contains(object):

    def __init__(self, value):
        self.value = value

    def __repr__(self):
        return self.value

    def __str__(self):
        return str(self.value)

    def __unicode__(self):
        return unicode(self.value)


contains = _Contains


def tag_by_alias(tag_name):
    return TAG_ALIASES.get(tag_name, tag_name)


def attribute_by_alias(atr_name):
    return change_name_from_python_to_html(
        ATTRIBUTE_ALIASES.get(atr_name, atr_name),
    )


def make_result(proxy, tag):
    def handle(**selector):
        query = [tag_by_alias(tag)]

        def get_format(value):
            if isinstance(value, contains):
                return u'[{}*="{}"]'
            return u'[{}="{}"]'

        query.extend(
            (
                get_format(val).format(attribute_by_alias(atr), val)
                for atr, val in selector.items()
            ),
        )

        return QueryResult(proxy, ''.join(query))

    return handle


def execute(proxy, css, get_all=False, disable_polling=False):
    logger.debug(u'CSS: {} Get all: {}'.format(css, 'Yes' if get_all else 'No'))

    proxy.reason_storage['last css query'] = css

    css_executors = {
        True: 'find_elements_by_css_selector',
        False: 'find_element_by_css_selector',
    }

    if disable_polling:
        with proxy.disable_polling():
            result = getattr(proxy, css_executors[bool(get_all)])(css)
    else:
        result = getattr(proxy, css_executors[bool(get_all)])(css)

    return result


class QueryResult(object):

    def __init__(self, proxy, css):
        self.__proxy = proxy
        self.__css = css

    def __getattr__(self, item):
        return getattr(
            self.__proxy.query.__class__(self.first()), item,
        )

    @property
    def exist(self):
        try:
            el = execute(self.__proxy, self.__css, disable_polling=True)

            if el:
                return True
            return False
        except WebDriverException:
            return False
        except BaseException:
            raise

    def wait(self, timeout=None):
        try:
            return waiting_for(
                lambda: self.exist,
                timeout=timeout or self.__proxy.config.POLLING_TIMEOUT,
            )
        except TimeoutException:
            raise TimeoutException(
                'Could not wait web element with css "{}"'.format(self.__css),
            )

    def get(self, index):
        try:
            return execute(self.__proxy, self.__css, get_all=True)[index]
        except IndexError:
            raise NoSuchElementException(
                'Result does not have element with index "{}". Css: "{}".'.format(
                    index, self.__css,
                ),
            )

    def first(self):
        return execute(self.__proxy, self.__css)

    def all(self):
        return execute(self.__proxy, self.__css, get_all=True)


class QueryProcessor(object):

    def __init__(self, proxy):
        self.__proxy = proxy

    def __getattr__(self, item):
        return make_result(self.__proxy, item)

    def __call__(self, proxy):
        return self.__class__(proxy)

    @property
    def proxy(self):
        return self.__proxy

    @property
    def driver(self):
        return self.__proxy.driver

    def from_object(self, obj):
        if not isinstance(obj, QueryObject):
            raise TypeError('"{}" is not QueryObject instance'.format(type(obj)))

        return self.__getattr__(obj.tag)(**obj.selector)



# -*- coding: utf-8 -*-

"""
A powerful, extensible and easy to use framework for test development

By Trifonov Mikhail

For support, use the https://github.com/trifonovmixail/seismograph/issues tracker
"""

from .case import skip
from .case import Case
from .case import flows
from .case import skip_if
from .case import assertion
from .case import CaseLayer
from .case import skip_unless
from .case import AssertionBase

from .suite import Suite
from .suite import SuiteLayer

from .program import main
from .program import Program
from .program import ProgramLayer

from .config import get_config_path_by_env

from .steps import step

from .datastructures import Context


__all__ = (
    'main',
    'skip',
    'step',
    'Case',
    'flows',
    'Suite',
    'skip_if',
    'Context',
    'Program',
    'assertion',
    'CaseLayer',
    'SuiteLayer',
    'skip_unless',
    'ProgramLayer',
    'AssertionBase',
    'get_config_path_by_env',
)


from .utils.pyv import check_py_version

check_py_version()

del check_py_version


import unittest
unittest.main



# -*- coding: utf-8 -*-

from . import fields
from .group import make_field
from .group import FieldsGroup
from .group import iter_fields
from .group import iter_invalid
from .group import iter_required
from .group import preserve_original


class UIForm(FieldsGroup):

    def submit(self):
        raise NotImplementedError(
            'Method "submit" not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )


__all__ = (
    'fields',
    'UIForm',
    'make_field',
    'FieldsGroup',
    'iter_fields',
    'iter_invalid',
    'iter_required',
    'preserve_original',
)



# -*- coding: utf-8 -*-

from . import steps
from . import runnable
from .utils import pyv


def format_reason(reason):
    return reason.__format_reason__()


def format_reason_to_output(reason):
    return reason.__format_reason_to_output__()


class Reason(object):

    def __init__(self, runnable_object, reason, config):
        self.__config = config
        self.__runnable_object = runnable_object
        self.__reason = pyv.unicode_string(reason)

    @property
    def runnable_object(self):
        return self.__runnable_object

    @property
    def reason(self):
        return self.__reason

    @property
    def config(self):
        return self.__config

    def __format_reason__(self):
        if steps.is_step_by_step_case(self.__runnable_object):
            if self.__runnable_object.__create_reason__:
                formatted_reason = (
                    steps.reason(self.__runnable_object),
                    runnable.reason(self.__runnable_object),
                    self.__reason,
                )
            else:
                formatted_reason = (
                    steps.reason(self.__runnable_object),
                    self.__reason,
                )
        elif self.__runnable_object.__create_reason__:
            formatted_reason = (
                runnable.reason(self.__runnable_object),
                self.__reason,
            )
        else:
            formatted_reason = (
                self.__reason,
            )

        return u''.join(formatted_reason)

    def __format_reason_to_output__(self):
        tmp = []

        runnable_repr = repr(self.__runnable_object)
        sep_line = ''.join(
            '=' for _ in pyv.xrange(
                len(runnable_repr),
            ),
        )

        tmp.append(sep_line)
        tmp.append(runnable_repr)
        tmp.append(sep_line)
        tmp.append(self.__reason)

        return u'\n'.join(tmp)


def create(runnable_object, reason, config=None):
    return Reason(runnable_object, reason, config)



# -*- coding: utf-8 -*-

import re

from .exceptions import RouterError
from .exceptions import RouteNotFound


def add_route(rule, cls):
    Router.add_rule(rule, cls)


class Router(object):

    __rules__ = {}

    def __init__(self, proxy):
        self.__proxy = proxy

    @classmethod
    def add_rule(cls, rule, cls_):
        cls.__rules__[re.compile(r'^{}$'.format(rule))] = cls_

    def get(self, path, go_to=True):
        for rule in self.__rules__:
            if rule.search(path) is not None:
                cls = self.__rules__[rule]
                page = cls(self.__proxy)
                break
        else:
            raise RouteNotFound(path)

        if go_to:
            self.go_to(path)

        return page

    def get_page(self, path):
        return self.get(path, go_to=False)

    def go_to(self, path):
        if not self.__proxy.config.PROJECT_URL:
            raise RouterError(
                'Can not go to the URL. Project URL is not set to config.',
            )

        self.__proxy.get(
            '{}{}'.format(
                self.__proxy.config.PROJECT_URL.strip('/'), path,
            ),
        )



# -*- coding: utf-8 -*-

import os
import imp
import time
import logging
from optparse import OptionGroup
from optparse import OptionParser
from importlib import import_module

from .exceptions import ConfigError
from .datastructures import DictObject


logger = logging.getLogger(__name__)


def create_option_parser():
    parser = OptionParser('seismograph <suites_path> [options]')

    result_group = OptionGroup(parser, 'Result options')
    result_group.add_option(
        '--xunit-report',
        dest='XUNIT_REPORT',
        default=None,
        help='Path to xml file to store the xunit report in.',
    )
    parser.add_option_group(result_group)

    console_group = OptionGroup(parser, 'Output options')
    console_group.add_option(
        '-v', '--verbose',
        action='store_true',
        dest='VERBOSE',
        default=False,
        help='Detail output.',
    )
    console_group.add_option(
        '-o', '--output',
        type=str,
        dest='OUTPUT',
        default=None,
        help='Redirect output to file.',
    )
    console_group.add_option(
        '--tree',
        dest='TREE',
        action='store_true',
        default=False,
        help='Print suites tree to console.',
    )
    console_group.add_option(
        '--no-color',
        dest='NO_COLOR',
        action='store_true',
        default=False,
        help='Not use color on output.',
    )
    parser.add_option_group(console_group)

    case_group = OptionGroup(parser, 'Case options')
    case_group.add_option(
        '--steps-log',
        dest='STEPS_LOG',
        action='store_true',
        default=False,
        help='Log steps history to output.',
    )
    case_group.add_option(
        '--flows-log',
        dest='FLOWS_LOG',
        action='store_true',
        default=False,
        help='Log flows to output.',
    )
    case_group.add_option(
        '--step-by-step',
        dest='STEP_BY_STEP',
        action='store_true',
        default=False,
        help='Do prompt before run test step.',
    )
    parser.add_option_group(case_group)

    run_group = OptionGroup(parser, 'Run options')
    run_group.add_option(
        '-t', '--test',
        action='append',
        dest='TESTS',
        default=[],
        help='Run these tests.',
    )
    run_group.add_option(
        '--include-regexp',
        dest='INCLUDE_SUITES_PATTERN',
        type=str,
        default=None,
        help='Regexp for allow registration suite by name.',
    )
    run_group.add_option(
        '--exclude-regexp',
        dest='EXCLUDE_SUITE_PATTERN',
        type=str,
        default=None,
        help='Regexp for not allow registration suite by name.',
    )
    run_group.add_option(
        '-x', '--stop',
        dest='STOP',
        action='store_true',
        default=False,
        help='Stop running tests after the first error or failure.',
    )
    run_group.add_option(
        '-r', '--repeat',
        dest='REPEAT',
        type=int,
        default=0,
        help='Num of tries to repeat run test',
    )
    run_group.add_option(
        '--random',
        dest='RANDOM',
        action='store_true',
        default=False,
        help='Random order when tests is running.',
    )
    run_group.add_option(
        '--random-seed',
        dest='RANDOM_SEED',
        default=time.time(),
        help='Num of tries to repeat run test',
    )
    run_group.add_option(
        '--no-skip',
        dest='NO_SKIP',
        action='store_true',
        default=False,
        help='Disable special handling of SkipTest exceptions.',
    )
    run_group.add_option(
        '--no-scripts',
        dest='NO_SCRIPTS',
        action='store_true',
        default=False,
        help='Ignore scripts on run.',
    )
    run_group.add_option(
        '--async-suites',
        type=int,
        dest='ASYNC_SUITES',
        default=0,
        help='Num suites to async run.',
    )
    run_group.add_option(
        '--async-tests',
        type=int,
        dest='ASYNC_TESTS',
        default=0,
        help='Num tests in suite to async run.',
    )
    run_group.add_option(
        '--mp-timeout',
        type=float,
        dest='MULTIPROCESSING_TIMEOUT',
        default=float(1800),
        help='Timeout to release and join multiprocessing process.',
    )
    run_group.add_option(
        '--gevent',
        dest='GEVENT',
        action='store_true',
        default=False,
        help='Use gevent groups to run. Allow for python 2 only.',
    )
    run_group.add_option(
        '--threading',
        dest='THREADING',
        action='store_true',
        default=False,
        help='Use threading groups to run.',
    )
    run_group.add_option(
        '--multiprocessing',
        dest='MULTIPROCESSING',
        action='store_true',
        default=False,
        help='Use multiprocessing groups to run.',
    )
    parser.add_option_group(run_group)

    return parser


def prepare_config(config):
    logging_settings = config.get('LOGGING_SETTINGS')

    if isinstance(logging_settings, dict):
        from logging.config import dictConfig
        dictConfig(logging_settings)

    if not config.GEVENT and not config.THREADING and (
            config.ASYNC_SUITES or config.ASYNC_TESTS):
        config.MULTIPROCESSING = True

    if (config.STEPS_LOG or config.FLOWS_LOG) and not config.VERBOSE:
        config.VERBOSE = True


def get_config_path_by_env(env_name, default=None, base_path=None):
    config_path = os.getenv(env_name, default)

    if base_path and config_path:
        config_path = '{}{}'.format(base_path, config_path)

    return config_path


def _load(obj, load_callable=True):
    def is_valid(atr):
        if load_callable:
            condition = not atr.startswith('_')
        else:
            condition = not atr.startswith('_') \
                and \
                not callable(getattr(obj, atr, None))

        return condition

    for atr in dir(obj):
        if is_valid(atr):
            yield (atr, getattr(obj, atr))


class Config(DictObject):

    def __init__(self, path=None, options=None):
        super(Config, self).__init__()

        if path:
            def is_import_path():
                is_dot_in_path = '.' in path
                is_py_ex = path.endswith('.py')

                return is_dot_in_path and not is_py_ex

            if is_import_path():
                self.from_module(path)
            elif path.endswith('.py'):
                self.from_py_file(path)

        if options:
            self.update(
                _load(options, load_callable=False)
            )

    def from_module(self, module):
        logger.debug('Init config from module "%s"', module)

        try:
            obj = import_module(module)
        except ImportError:
            raise ImportError('Config {} not found'.format(module))

        self.update(_load(obj))

        return self

    def from_py_file(self, file_path):
        logger.debug('Init config from py file "%s"', file_path)

        if not os.path.isfile(file_path):
            raise ConfigError('config file does not exist at path "{}"'.format(file_path))

        elif not file_path.endswith('.py'):
            raise ConfigError('config file is not python file')

        module = imp.new_module(file_path.rstrip('.py'))
        module.__file__ = file_path

        try:
            execfile(file_path, module.__dict__)
        except IOError as e:
            e.strerror = 'Unable to load file "{}"'.format(e.strerror)
            raise

        self.update(_load(module))

        return self



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import os
import logging
from functools import wraps

try:
    from httplib import HTTPException
except ImportError:  # please python 3
    from http.client import HTTPException

from selenium.common.exceptions import WebDriverException
from selenium.common.exceptions import StaleElementReferenceException
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

from ... import case
from ... import steps
from . import drivers
from ... import suite
from ... import runnable
from .proxy import WebDriverProxy
from ...tools import create_reason
from ...utils.common import waiting_for
from .exceptions import SeleniumExError
from ...exceptions import TimeoutException
from .tools import create_screen_file_name


logger = logging.getLogger(__name__)


EX_NAME = 'selenium'

GET_DRIVER_TIMEOUT = 10

DEFAULT_WINDOW_SIZE = None
DEFAULT_POLLING_TIMEOUT = 30
DEFAULT_MAXIMIZE_WINDOW = True
DEFAULT_DRIVER = drivers.CHROME

DRIVER_TO_CAPABILITIES = {
    drivers.OPERA: DesiredCapabilities.OPERA,
    drivers.CHROME: DesiredCapabilities.CHROME,
    drivers.FIREFOX: DesiredCapabilities.FIREFOX,
    drivers.PHANTOMJS: DesiredCapabilities.PHANTOMJS,
    drivers.IE: DesiredCapabilities.INTERNETEXPLORER,
}


def get_capabilities(driver_name):
    """
    Get capabilities of driver

    :param driver_name: driver name
    :type driver_name: str
    """
    try:
        return DRIVER_TO_CAPABILITIES[driver_name]
    except KeyError:
        raise SeleniumExError(
            'Capabilities for driver "{}" is not found'.find(driver_name),
        )


def patch(f):
    """
    Setup config to driver and apply settings.
    Wrap driver in proxy object.
    """
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        driver = f(self, *args, **kwargs)

        browser = WebDriverProxy(
            driver,
            config=BrowserConfig(self, driver),
        )

        self._Selenium__browser = browser

        return browser

    return wrapper


class BrowserConfig(object):
    """
    Configuration for WerDriver instance
    """

    def __init__(self, selenium, browser):
        self.__browser = browser

        self.__WINDOW_SIZE = None
        self.WINDOW_SIZE = selenium.config.get('WINDOW_SIZE')

        self.__MAXIMIZE_WINDOW = False
        self.MAXIMIZE_WINDOW = selenium.config.get('MAXIMIZE_WINDOW')

        self.PROJECT_URL = selenium.config.get('PROJECT_URL')
        self.POLLING_TIMEOUT = selenium.config.get('POLLING_TIMEOUT')

    @property
    def WINDOW_SIZE(self):
        return self.__WINDOW_SIZE

    @WINDOW_SIZE.setter
    def WINDOW_SIZE(self, value):
        if value:
            self.__WINDOW_SIZE = value
            self.__browser.set_window_size(
                *self.WINDOW_SIZE
            )

    @property
    def MAXIMIZE_WINDOW(self):
        return self.__MAXIMIZE_WINDOW

    @MAXIMIZE_WINDOW.setter
    def MAXIMIZE_WINDOW(self, value):
        if value and not self.__WINDOW_SIZE:
            self.__MAXIMIZE_WINDOW = value
            self.__browser.maximize_window()


class Selenium(object):

    def __init__(self, config):
        self.__config = config

        self.__browser = None
        self.__browser_name = None

    def __enter__(self):
        self.start()
        return self.__browser

    def __exit__(self, *args, **kwargs):
        self.stop()

    @property
    def config(self):
        return self.__config

    @property
    def browser(self):
        return self.__browser

    @property
    def browser_name(self):
        return self.__browser_name

    def start(self, *args, **kwargs):
        if self.__browser:
            return

        self.get_browser(*args, **kwargs)

    def stop(self):
        if not self.__browser:
            return

        self.__browser.quit()
        self.__browser = None

    @patch
    def remote(self, driver_name):
        remote_config = self.__config.get('REMOTE')

        if not remote_config:
            raise SeleniumExError('settings of remote not found in config')

        logger.debug('Remote config: {}'.format(str(remote_config)))

        options = remote_config.get('OPTIONS', {})
        capabilities = get_capabilities(driver_name)
        capabilities.update(
            remote_config['CAPABILITIES'][driver_name],
        )

        self.__browser = drivers.RemoteWebDriver(
            desired_capabilities=capabilities,
            **options
        )

        return self.__browser

    @patch
    def ie(self):
        ie_config = self.__config.get('IE')

        if not ie_config:
            raise SeleniumExError('settings of ie browser not found in config')

        logger.debug('Ie config: {}'.format(str(ie_config)))

        self.__browser = drivers.IeWebDriver(**ie_config)

        return self.__browser

    @patch
    def chrome(self):
        """
        :rtype: selenium.webdriver.chrome.webdriver.WebDriver
        """
        chrome_config = self.__config.get('CHROME')

        if not chrome_config:
            raise SeleniumExError('settings of chrome browser not found in config')

        logger.debug('Chrome config: {}'.format(str(chrome_config)))

        self.__browser = drivers.ChromeWebDriver(**chrome_config)

        return self.__browser

    @patch
    def firefox(self):
        firefox_config = self.__config.get('FIREFOX', {})

        logger.debug('Firefox config: {}'.format(str(firefox_config)))

        self.__browser = drivers.FirefoxWebDriver(**firefox_config)

        return self.__browser

    @patch
    def phantomjs(self):
        phantom_config = self.__config.get('PHANTOMJS')

        if not phantom_config:
            raise SeleniumExError('settings of phantom js not found in config')

        logger.debug('PhantomJS config: {}'.format(str(phantom_config)))

        self.__browser = drivers.PhantomJSWebDriver(**phantom_config)

        return self.__browser

    @patch
    def opera(self):
        opera_config = self.__config.get('OPERA')

        if not opera_config:
            raise SeleniumExError('settings of opera browser not found in config')

        logger.debug('Opera config: {}'.format(str(opera_config)))

        self.__browser = drivers.OperaWebDriver(**opera_config)

        return self.__browser

    def _get_local_driver(self, driver_name):
        method = getattr(self, driver_name, None)

        if method:
            return method()

        raise SeleniumExError(
            'Incorrect browser name "{}"'.format(driver_name),
        )

    def get_browser(self, browser_name=None, timeout=None):
        if self.__browser:
            return self.__browser

        self.__browser_name = (
            browser_name or self.__config.get('DEFAULT_BROWSER', drivers.FIREFOX)
        ).lower()

        def get_browser(func, *args, **kwargs):
            try:
                return func(*args, **kwargs)
            except (IOError, OSError, HTTPException, WebDriverException):
                return None

        if self.__config.get('USE_REMOTE', False):
            driver = waiting_for(
                lambda: get_browser(self.remote, self.__browser_name),
                timeout=timeout or GET_DRIVER_TIMEOUT,
            )
        else:
            driver = waiting_for(
                lambda: get_browser(self._get_local_driver, self.__browser_name),
                timeout=timeout or GET_DRIVER_TIMEOUT,
            )

        return driver


class SeleniumAssertion(case.AssertionBase):

    DEFAULT_TIMEOUT = 3

    def one_of_texts_in_page(self, browser, texts, msg=None):
        page_text = browser.text
        error_message = u'Not of those texts was not found in page "{}". Texts: [{}]'

        for text in texts:
            if text in page_text:
                break
        else:
            error_message = error_message.format(
                browser.current_url,
                u', '.join(texts),
            )
            self.fail(msg or error_message)

    def text_in_page(self, browser, text_or_texts, timeout=None, msg=None):
        error_message = u'Text "{}" not found in page "{}"'

        def check_text(txt):
            try:
                return txt in browser.text
            except (HTTPException, StaleElementReferenceException):
                return False

        if isinstance(text_or_texts, (list, tuple)):
            for t in text_or_texts:
                try:
                    waiting_for(
                        check_text,
                        timeout=timeout or browser.config.POLLING_TIMEOUT or self.DEFAULT_TIMEOUT,
                        args=(t,),
                    )
                except TimeoutException:
                    self.fail(
                        msg or error_message.format(
                            t, browser.current_url,
                        ),
                    )
        else:
            try:
                waiting_for(
                    check_text,
                    timeout=timeout or browser.config.POLLING_TIMEOUT or self.DEFAULT_TIMEOUT,
                    args=(text_or_texts,),
                )
            except TimeoutException:
                self.fail(
                    msg or error_message.format(
                        text_or_texts, browser.current_url,
                    ),
                )

    def web_element_exist(self, browser, query_object, timeout=None, msg=None):
        try:
            waiting_for(
                lambda: browser.query.form_object(query_object).exist,
                timeout=timeout or browser.config.POLLING_TIMEOUT or self.DEFAULT_TIMEOUT,
            )
        except TimeoutException:
            self.fail(msg or u'Web element was not found')

    def web_element_not_exist(self, browser, query_object, timeout=None, msg=None):
        try:
            waiting_for(
                lambda: not browser.query.form_object(query_object).exist,
                timeout=timeout or browser.config.POLLING_TIMEOUT or self.DEFAULT_TIMEOUT,
            )
        except TimeoutException:
            self.fail(msg or u'Web element was found')

    def content_in_web_element_exist(self, browser, query_object, timeout=None, msg=None):
        try:
            waiting_for(
                lambda: len(browser.query.form_object(query_object).first().text) > 0,
                timeout=timeout or browser.config.POLLING_TIMEOUT or self.DEFAULT_TIMEOUT,
            )
        except TimeoutException:
            self.true(msg or 'Content does not exist in element')


assertion = SeleniumAssertion()


def _get_selenium_from_case(self):
    return self._SeleniumCase__selenium


def case_of_browsers(*browsers):
    def wrapper(cls):
        setattr(cls, '__browsers__', browsers)
        return cls
    return wrapper


def inject_driver(f):
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        return f(
            self, _get_selenium_from_case(self).browser, *args, **kwargs
        )
    return wrapper


class SeleniumCaseLayer(case.CaseLayer):

    def on_require(self, require):
        if EX_NAME not in require:
            require.append(EX_NAME)

    def on_setup(self, case):
        if not case.__browsers__:
            _get_selenium_from_case(case).start()

    def on_teardown(self, case):
        if not case.__browsers__:
            _get_selenium_from_case(case).stop()


class SeleniumCase(case.Case):

    __browsers__ = None
    __create_reason__ = True
    __layers__ = (SeleniumCaseLayer(), )
    __assertion_class__ = SeleniumAssertion

    def __init__(self, *args, **kwargs):
        # do it latter
        kwargs.update(use_flows=False)

        super(SeleniumCase, self).__init__(*args, **kwargs)

        if self.config.SELENIUM_BROWSERS:
            self.__browsers__ = self.config.SELENIUM_BROWSERS

        self.__selenium = self.ext('selenium')

        if steps.is_step_by_step_case(self):
            step_methods = []

            for step_method in steps.get_step_methods(self):
                step_methods.append(inject_driver(step_method))

            setattr(self.__class__, steps.STEPS_STORAGE_ATTRIBUTE_NAME, step_methods)
        else:
            method = getattr(self.__class__, runnable.method_name(self))
            setattr(self.__class__, runnable.method_name(self), inject_driver(method))

            case.apply_flows(self)

    def __reason__(self):
        reason_args = []

        for k, v in self.__selenium.browser.reason_storage.items():
            reason_args.append('{}: {}'.format(k, v))

        screen_url = self.__selenium.config.get('SCREEN_URL', None)
        screen_path = self.__selenium.config.get('SCREEN_PATH', None)

        if screen_path:
            file_name = create_screen_file_name()
            screen_path = os.path.join(screen_path, file_name)

            self.__selenium.browser.save_screenshot(screen_path)

            if screen_url:
                reason_args.append(
                    u'screen url: {}{}'.format(screen_url, file_name),
                )
            else:
                reason_args.append('screen path: {}'.format(screen_path))

        return create_reason(
            'Selenium',
            'info from selenium extension',
            'browser name: {}'.format(self.__selenium.browser_name),
            *reason_args
        )

    def __repeat__(self):
        if self.__browsers__:
            for browser_name in self.__browsers__:
                self.__selenium.start(browser_name)
                try:
                    yield
                finally:
                    self.__selenium.stop()
        else:
            yield


class SeleniumSuiteLayer(suite.SuiteLayer):

    def on_require(self, require):
        if EX_NAME not in require:
            require.append(EX_NAME)


class SeleniumSuite(suite.Suite):

    __case_class__ = SeleniumCase
    __layers__ = (SeleniumSuiteLayer(), )



# -*- coding: utf-8 -*-

"""
Module for supporting py 2-3 versions.
"""

import sys
import types

from ..exceptions import PyVersionError


IS_PYTHON_3 = sys.version_info >= (3, 0)
IS_PYTHON_2 = not IS_PYTHON_3


def check_py_version():
    error_mess = 'Unsupported py version. Need python 2.7 and greater.'

    if not IS_PYTHON_3 and not IS_PYTHON_2:
        raise PyVersionError(error_mess)

    if sys.version_info < (2, 7):
        raise PyVersionError(error_mess)


def check_gevent_supported():
    if IS_PYTHON_3:
        raise PyVersionError('gevent lib not supported with python 3')


if IS_PYTHON_2:
    basestring = basestring
elif IS_PYTHON_3:
    basestring = str


if IS_PYTHON_2:
    unicode = unicode
elif IS_PYTHON_3:
    unicode = str


if IS_PYTHON_2:
    xrange = xrange
elif IS_PYTHON_3:
    xrange = range


if IS_PYTHON_2:
    reduce = reduce
elif IS_PYTHON_3:
    from functools import reduce
    reduce = reduce


def get_exc_message(error):
    if hasattr(error, 'message'):
        return error.message

    return u''.join((unicode_string(m) for m in error.args))


def get_func_name(func):
    if hasattr(func, '__func__'):
        return func.__func__.__name__

    return func.__name__


def is_class_type(obj):
    if IS_PYTHON_2:
        return isinstance(obj, (type, types.ClassType))

    return isinstance(obj, type)


def unicode_string(string):
    try:
        return unicode(string)
    except UnicodeDecodeError as e:
        try:
            return unicode(string.decode('utf-8'))
        except AttributeError:
            raise e



# -*- coding: utf-8 -*-

from functools import wraps
from contextlib import contextmanager

from .utils import pyv
from .utils.common import MPSupportedValue


def reason(runnable):
    return runnable.__reason__()


def stopped_on(runnable, method_name=None):
    if method_name:
        runnable._stopped_on = method_name
        return method_name

    return runnable.__stopped_on__()


def is_run(runnable):
    return runnable.__is_run__()


def is_mount(runnable):
    assert issubclass(runnable.__class__, MountObjectMixin), \
        '"{}" is not mount object'.format(runnable.__class__)
    return runnable.__is_mount__()


def is_build(runnable):
    assert issubclass(runnable.__class__, BuildObjectMixin), \
        '"{}" is not build object'.format(runnable.__class__)
    return runnable.__is_build__()


def method_name(runnable):
    return runnable.__method_name__()


def class_name(runnable):
    return runnable.__class_name__()


def run_method(f):
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        assert is_run(self), 'Can not call "{}" of "{}.{}". Should be run.'.format(
            pyv.get_func_name(f), self.__class__.__module__, self.__class__.__name__,
        )
        return f(self, *args, **kwargs)
    return wrapper


def mount_method(f):
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        assert is_mount(self), 'Can not call "{}" of "{}.{}". Should be mount.'.format(
            pyv.get_func_name(f), self.__class__.__module__, self.__class__.__name__,
        )
        return f(self, *args, **kwargs)
    return wrapper


def build_method(f):
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        assert is_build(self), 'Can not call "{}" of "{}.{}". Should be built.'.format(
            pyv.get_func_name(f), self.__class__.__module__, self.__class__.__name__,
        )
        return f(self, *args, **kwargs)
    return wrapper


class RunnableObject(object):

    __create_reason__ = False

    def __init__(self):
        self.__id = id(self)
        self.__stopped_on = MPSupportedValue(
            method_name(self),
        )

    def __call__(self, *args, **kwargs):
        return self.run(*args, **kwargs)

    def __repr__(self):
        class_path = '{}.{}'.format(
            self.__class__.__module__, self.__class__.__name__,
        )
        return '<{} method_name={} stopped_on={}>'.format(
            class_path, method_name(self), stopped_on(self),
        )

    @property
    def id(self):
        return self.__id

    @property
    def _stopped_on(self):
        return self.__stopped_on.value

    @_stopped_on.setter
    def _stopped_on(self, value):
        self.__stopped_on.value = value

    def __is_run__(self):
        raise NotImplementedError(
            'Method "__is_run__" not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )

    def __method_name__(self):
        return 'run'

    def __stopped_on__(self):
        return self._stopped_on

    def __class_name__(self):
        return '{}.{}'.format(
            self.__class__.__module__, self.__class__.__name__,
        )

    def __reason__(self):
        return 'Your reason can be here. This is from "{}.{}.__reason__" method.\n'.format(
            self.__class__.__module__, self.__class__.__name__,
        )

    def support_mp(self, stopped_on=None):
        if stopped_on:
            self.__stopped_on.set_mp(stopped_on)

    def run(self, *args, **kwargs):
        raise NotImplementedError(
            'Method "run" not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )


class BuildObjectMixin(object):

    def __is_build__(self):
        raise NotImplementedError(
            'Method "__is_mount__" not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )


class MountObjectMixin(object):

    def __is_mount__(self):
        raise NotImplementedError(
            'Method "__is_mount__" not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )


class ContextOfRunnableObject(object):

    @contextmanager
    def __call__(self, runnable):
        self.start_context(runnable)
        try:
            yield
        finally:
            if stopped_on(runnable) != 'start_context':
                self.stop_context(runnable)

    @property
    def layers(self):
        raise NotImplementedError(
            'Property "layers" is not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )

    @property
    def setup_callbacks(self):
        raise NotImplementedError(
            'Property "setup_callbacks" is not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )

    @property
    def teardown_callbacks(self):
        raise NotImplementedError(
            'Property "teardown_callbacks" is not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )

    def start_context(self, obj):
        raise NotImplementedError(
            'Method "start_context" is not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )

    def stop_context(self, obj):
        raise NotImplementedError(
            'Method "stop_context" is not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )


class LayerOfRunnableObject(object):

    def __init__(self):
        self.enabled = True


class RunnableGroup(RunnableObject):

    def __init__(self, objects, config):
        super(RunnableGroup, self).__init__()

        self.__objects = objects
        self.__config = config

        self._is_run = False

    def __is_run__(self):
        return self._is_run

    @property
    def config(self):
        return self.__config

    @property
    def objects(self):
        return self.__objects


class Script(RunnableObject):

    __run_point__ = None

    def __init__(self, program):
        super(Script, self).__init__()

        self.__program = program

    @property
    def config(self):
        return self.__program.config



# -*- coding: utf-8 -*-

import os
import time
import logging
from contextlib import contextmanager

import requests
from flask import Flask as _Flask

from . import patch
from . import mock as _mock
from .exceptions import MockServerError
from .exceptions import MockServerClientError


APP_NAME = 'mocker'

DEFAULT_PORT = 5000
DEFAULT_HOST = '127.0.0.1'

DEFAULT_START_SERVER_TIMEOUT = 10


# Patch flask
Flask = type(
    'Flask',
    (_Flask, ),
    {
        'add_url_rule': patch.add_url_rule,
    },
)


def set_log_level_for_dependency(debug):
    loggers = (
        logging.getLogger('flask'),
        logging.getLogger('werkzeug'),
    )

    handlers = {
        False: logging.NullHandler,
        True: logging.StreamHandler,
    }

    for logger in loggers:
        logger.handlers = []
        logger.handlers.append(handlers[bool(debug)]())
        logger.setLevel(logging.INFO if debug else logging.ERROR)


class MockServerClient(object):

    ALLOW_HTTP_METHODS = (
        'get',
        'put',
        'head',
        'post',
        'patch',
        'delete',
        'options',
    )

    def __init__(self, server):
        self.__server = server

    def __getattr__(self, http_method):
        if http_method not in self.ALLOW_HTTP_METHODS:
            raise MockServerClientError(
                'Incorrect http method for request: "{}"'.format(http_method),
            )

        method = getattr(requests, http_method)

        def method_wrapper(path, *args, **kwargs):
            return method(
                'http://{}:{}{}'.format(
                    self.__server.host,
                    self.__server.port,
                    path,
                ),
                *args, **kwargs
            )

        return method_wrapper



class BaseMockServer(object):

    __mock_class__ = _mock.BaseMock
    __file_extensions__ = ('.resp', )

    def __init__(self,
                 mock_path=None,
                 host=None,
                 port=None,
                 mocks=None,
                 debug=False,
                 gevent=False,
                 threading=False,
                 multiprocessing=False):
        set_log_level_for_dependency(debug)

        self.__app = Flask(APP_NAME)
        self.__app.debug = debug

        self.__debug = debug
        self.__host = host or DEFAULT_HOST
        self.__port = port or DEFAULT_PORT

        self._gevent = gevent
        self._threading = threading
        self._multiprocessing = multiprocessing

        if not threading and not gevent and not multiprocessing:
            self._threading = True

        self._process = None
        self._started = False
        self._client = MockServerClient(self)

        if mock_path:
            self.scan_dir(mock_path)

        if mocks:
            for mock in mocks:
                self.add_mock(mock)

    def __call__(self, *args, **kwargs):
        return self.wsgi_handler(*args, **kwargs)

    @property
    def _stopped(self):
        return not self._started

    @property
    def views(self):
        return self.__app.view_functions

    @property
    def host(self):
        return self.__host

    @property
    def port(self):
        return self.__port

    @property
    def client(self):
        return self._client

    @property
    def debug(self):
        return self.__debug

    @debug.setter
    def debug(self, value):
        self.__debug = value
        self.__app.debug = value
        set_log_level_for_dependency(value)

    @property
    def mocks(self):
        return (
            v for v in self.views.values()
            if isinstance(v, _mock.BaseMock)
        )

    @property
    def wsgi_handler(self):
        return self.__app.wsgi_app

    def scan_dir(self, mock_path):
        for name in os.listdir(mock_path):
            path = os.path.join(mock_path, name)

            if any(name.endswith(ex) for ex in self.__file_extensions__):
                with open(path, 'r') as fp:
                    mock = self.__mock_class__(path, None)
                    _mock.on_file(mock, fp)
                self.add_mock(mock)

            if os.path.isdir(path):
                self.scan_dir(path)

    def add_url_rule(self, rule, endpoint=None, view_func=None, **options):
        self.__app.add_url_rule(
            rule, endpoint=endpoint, view_func=view_func, **options
        )

    def delete_mock(self, mock):
        try:
            del self.views[mock.endpoint]
        except KeyError:
            raise MockServerError(
                'Can\' delete mock "{}" because not exist'.format(mock),
            )

    def add_mock(self, mock, rewrite=False):
        """
        :type mock: BaseMock
        :type rewrite: bool
        """
        need_restart = self._started and self._multiprocessing

        if need_restart:
            self.stop()

        try:
            if rewrite:
                self.delete_mock(mock)
                self.add_mock(mock)
            else:
                self.add_url_rule(
                    mock.url_rule,
                    view_func=mock,
                    endpoint=mock.endpoint,
                    methods=[mock.http_method],
                )
        finally:
            if need_restart:
                self.start()

    @contextmanager
    def mock(self, mock):
        """
        :type mock: BaseMock
        """
        for old_mock in self.mocks:
            if old_mock == mock:
                with mock.as_(old_mock):
                    self.add_mock(mock, rewrite=True)
                try:
                    yield
                finally:
                    self.add_mock(old_mock, rewrite=True)
                break
        else:
            self.add_mock(mock)
            try:
                yield
            finally:
                _mock.abort(mock, 404)

    def run(self):
        self.__app.run(host=self.__host, port=self.__port)

    def serve_forever(self):
        if self._gevent:
            from gevent.wsgi import WSGIServer

            WSGIServer(
                (self.__host, self.__port), self,
                log='default' if self.__debug else None,
            ).serve_forever()
        if self._threading:
            from werkzeug.serving import ThreadedWSGIServer

            ThreadedWSGIServer(self.__host, self.__port, self).serve_forever()
        if self._multiprocessing:
            from werkzeug.serving import ForkingWSGIServer

            ForkingWSGIServer(self.__host, self.__port, self).serve_forever()
        else:
            self.run()

    def waiting_for_ready(self, timeout=DEFAULT_START_SERVER_TIMEOUT):
        if self._stopped:
            raise MockServerError('Mock server is not started')

        is_error = True
        t_start = time.time()

        while time.time() <= t_start + timeout:
            if not is_error:
                break

            try:
                self._client.get('/')
                is_error = False
            except requests.ConnectionError:
                pass

        if is_error:
            raise MockServerError(
                'Mock server have not been started for "{}" sec.'.format(timeout),
            )

    def start(self):
        if self._started:
            return

        if self._gevent:
            from .run_server.gevent import RunServerInGreenlet

            self._process = RunServerInGreenlet(self)
        elif self._threading:
            from werkzeug.serving import ThreadedWSGIServer
            from .run_server.threading import RunServerInThread

            self._process = RunServerInThread(self, ThreadedWSGIServer)
        elif self._multiprocessing:
            from werkzeug.serving import ForkingWSGIServer
            from .run_server.threading import RunServerInThread

            self._process = RunServerInThread(self, ForkingWSGIServer)
        else:
            raise MockServerError(
                'Can not start mock server. Don\'t know how :(.'
            )

        self._process.start()

        self._started = True
        self.waiting_for_ready()

    def restart(self):
        self.stop()
        self.start()

    def stop(self):
        if self._stopped:
            return

        if self._gevent:
            self._process.kill()
        elif self._threading:
            self._process.stop()
        elif self._multiprocessing:
            self._process.stop()
        else:
            raise MockServerError(
                'Can not stop mock server. Don\'t know how :(.'
            )

        self._process.join()

        self._process = None
        self._started = False



# -*- coding: utf-8 -*-

import os
import sys
import time
import logging
from random import randint
from importlib import import_module

from .exceptions import LoaderError


logger = logging.getLogger(__name__)


TEST_NAME_PREFIX = 'test'
DEFAULT_TEST_NAME = 'test'


def check_path_is_exist(path):
    if not os.path.exists(path):
        raise LoaderError('Dir "{}" is not exist'.format(path))


def is_package(path):
    return os.path.isfile(
        os.path.join(path, '__init__.py'),
    )


def is_py_module(file_name):
    return not file_name.startswith('_') \
        and file_name.endswith('.py')


def load_module(module_name, package=None):
    module_name = '{}{}'.format(
        package + '.' if package else '', module_name,
    )

    # Can be conflict with global name
    # We're obliged to talk about this
    if module_name in sys.modules:
        raise LoaderError(
            'Module "{}" already exit in program context. {}.'.format(
                module_name, sys.modules[module_name],
            ),
        )

    module = import_module(module_name)

    load_session = str(time.time() + randint(0, 1000))
    system_name = '{}_{}'.format(module_name, load_session)

    del sys.modules[module_name]
    sys.modules[system_name] = module

    return module


def load_test_names_from_case(
        cls,
        test_name_prefix=None,
        default_test_name=None):
    def is_test_name(name):
        return name.startswith((test_name_prefix or TEST_NAME_PREFIX)) \
            or \
            name == (default_test_name or DEFAULT_TEST_NAME)

    for name in sorted(dir(cls)):
        if is_test_name(name):
            yield name


def load_tests_from_case(
        test_case_class,
        config=None,
        box_class=None,
        method_name=None,
        test_name_prefix=None,
        default_test_name=None):
    logger.debug('Load test from test case: "%s"', test_case_class.__name__)

    if method_name:
        for name in filter(lambda n: n == method_name, dir(test_case_class)):
            case = test_case_class(name, config=config)
            if box_class:
                yield box_class((case, ))
            else:
                yield case
            raise StopIteration
        else:
            raise LoaderError(
                'Test "{}" is not found on "{}"'.format(
                    method_name, test_case_class.__name__,
                ),
            )
    else:
        names = load_test_names_from_case(
            test_case_class,
            test_name_prefix=(test_name_prefix or TEST_NAME_PREFIX),
            default_test_name=(default_test_name or DEFAULT_TEST_NAME),
        )

        if box_class:
            cases = []

            for name in names:
                cases.append(
                    test_case_class(name, config=config)
                )

            yield box_class(cases)
        else:
            for name in names:
                yield test_case_class(name, config=config)


def load_suite_by_name(name, suites):
    logger.debug('Load suite "%s" from list', name)

    for suite in filter(lambda s: s.name == name, suites):
        return suite
    else:
        raise LoaderError(
            'Suite "{}" is not found'.format(name),
        )


def load_case_from_suite(class_name, suite):
    logger.debug('Load test case "%s" from suite "%s"', class_name, suite.name)

    for case_cls in filter(lambda c: c.__name__ == class_name, suite.cases):
        return case_cls
    else:
        raise LoaderError(
            'Test case "{}" is not found'.format(class_name),
        )


def load_suites_from_module(module, suite_class):
    for attribute in dir(module):
        value = getattr(module, attribute, None)
        if isinstance(value, suite_class):
            yield value


def load_suites_from_path(path_to_dir, suite_class, package=None, recursive=True):
    check_path_is_exist(path_to_dir)

    lst_dir = os.listdir(path_to_dir)
    full_path = lambda *n: os.path.join(path_to_dir, *n)

    modules = (n.replace('.py', '') for n in lst_dir if is_py_module(n))

    for module_name in modules:
        module = load_module(module_name, package=package)

        for suite in load_suites_from_module(module, suite_class):
            yield suite

    if recursive:
        packs = (n for n in lst_dir if is_package(full_path(n)))

        for pack in packs:
            if package:
                pack = '{}.{}'.format(package, pack)

            for suite in load_suites_from_path(
                    full_path(pack), suite_class, package=pack):
                yield suite



# -*- coding: utf-8 -*-

"""
Module for configuration program context
"""

import seismograph.case as _case
import seismograph.xunit as _xunit
import seismograph.suite as _suite
import seismograph.config as _config
import seismograph.reason as _reason
import seismograph.result as _result
import seismograph.loader as _loader
import seismograph.program as _program
import seismograph.runnable as _runnable
import seismograph.utils.common as _common_utils


shared_data = _program.Program.shared_data
shared_extension = _program.Program.shared_extension


def match_suite_to_layer(cls, layer):
    _suite.MATCH_SUITE_TO_LAYER[cls] = layer


def match_case_to_layer(cls, layer):
    _case.MATCH_CASE_TO_LAYER[cls] = layer


def set_default_case_layers(*layers):
    _case.DEFAULT_LAYERS.extend(layers)


def set_default_suite_layers(*layers):
    _suite.DEFAULT_LAYERS.extend(layers)


def set_default_program_layers(*layers):
    _program.DEFAULT_LAYERS.extend(layers)


def configure(
        wait_sleep=None,
        reason_class=None,
        wait_timeout=None,
        config_class=None,
        result_class=None,
        round_runtime=None,
        config_env_name=None,
        test_name_prefix=None,
        case_group_class=None,
        suite_group_class=None,
        default_test_name=None,
        case_context_class=None,
        suite_context_class=None,
        skip_attribute_name=None,
        result_marker_class=None,
        program_context_class=None,
        skip_why_attribute_name=None,
        use_static_test_functions=False):
    if round_runtime:
        assert type(round_runtime) == int
        _xunit.ROUND_RUNTIME = round_runtime

    if use_static_test_functions:
        _case.Case.__static__ = True

    if config_env_name:
        _program.CONFIG_ENV_NAME = config_env_name

    if wait_sleep:
        assert type(wait_sleep) in (int, float)
        _common_utils.WAITING_FOR_SLEEP = wait_sleep

    if test_name_prefix:
        _loader.TEST_NAME_PREFIX = test_name_prefix

    if default_test_name:
        _loader.DEFAULT_TEST_NAME = default_test_name

    if skip_attribute_name:
        _case.SKIP_ATTRIBUTE_NAME = skip_attribute_name

    if wait_timeout:
        assert type(wait_timeout) in (int, float)
        _common_utils.WAITING_FOR_TIMEOUT = wait_timeout

    if result_class:
        assert issubclass(result_class, _result.Result)
        _program.Program.__result_class__ = result_class

    if config_class:
        assert issubclass(config_class, _config.Config)
        _program.Program.__config_class__ = config_class

    if skip_why_attribute_name:
        _case.SKIP_WHY_ATTRIBUTE_NAME = skip_why_attribute_name

    if case_group_class:
        assert issubclass(case_group_class, _runnable.RunnableGroup)
        _suite.Suite.__case_group_class__ = case_group_class

    if result_marker_class:
        assert issubclass(result_marker_class, _result.ResultMarkers)
        _result.Result.__marker_class__ = result_marker_class

    if case_context_class:
        assert issubclass(case_context_class, _case.CaseContext)
        _case.CaseContext = case_context_class

    if reason_class:
        assert issubclass(reason_class, _reason.Reason)
        _reason.Reason = reason_class

    if suite_context_class:
        assert issubclass(suite_context_class, _suite.SuiteContext)
        _suite.SuiteContext = suite_context_class

    if program_context_class:
        assert issubclass(program_context_class, _program.ProgramContext)
        _program.ProgramContext = program_context_class

    if suite_group_class:
        assert issubclass(suite_group_class, _runnable.RunnableGroup)
        _program.Program.__suite_group_class__ = suite_group_class



# -*- coding: utf-8 -*-

from optparse import OptionGroup

from ...extensions import BaseExtension

from .selenium import EX_NAME
from .selenium import Selenium
from .selenium import assertion
from .selenium import inject_driver
from .selenium import case_of_browsers
from .selenium import SeleniumAssertion
from .selenium import SeleniumCase as Case
from .selenium import SeleniumSuite as Suite

from . import forms

from .query import QueryObject

from .router import add_route

from .page_object import Page
from .page_object import PageObject
from .page_object import PageObjectProxy


query = QueryObject


class SeleniumExtension(BaseExtension):

    __ex_name__ = EX_NAME
    __config_key__ = 'SELENIUM_EX'

    @staticmethod
    def __add_options__(parser):
        group = OptionGroup(parser, 'Selenium extension options')

        group.add_option(
            '--selenium-browser',
            dest='SELENIUM_BROWSERS',
            action='append',
            default=[],
            help='Browser names for run cases.',
        )
        group.add_option(
            '--selenium-project-url',
            dest='SELENIUM_PROJECT_URL',
            default=None,
            help='Base URL of your project.',
        )
        group.add_option(
            '--selenium-polling',
            dest='SELENIUM_POLLING',
            type=float,
            default=None,
            help='Polling timeout.',
        )
        group.add_option(
            '--selenium-window-size',
            dest='SELENIUM_WINDOW_SIZE',
            default=None,
            help='Window size. For example: 340x480',
        )

        parser.add_option_group(group)

    def __install__(self, program):
        config = program.config.get(self.__config_key__, {})

        if program.config.SELENIUM_PROJECT_URL:
            config['PROJECT_URL'] = program.config.SELENIUM_PROJECT_URL

        if program.config.SELENIUM_POLLING:
            config['POLLING_TIMEOUT'] = program.config.SELENIUM_POLLING

        if program.config.SELENIUM_WINDOW_SIZE:
            w, h = program.config.SELENIUM_WINDOW_SIZE.split('x')
            config['WINDOW_SIZE'] = (int(w), int(h))

        program.shared_extension(
            self.__ex_name__, Selenium, args=(config, ),
        )

    def __call__(self):
        raise TypeError(
            '"{}" is not callable'.format(
                self.__class__.__name__,
            ),
        )


__all__ = (
    'Case',
    'forms',
    'query',
    'Suite',
    'Router',
    'Selenium',
    'add_route',
    'assertion',
    'QueryObject',
    'SeleniumCase',
    'inject_driver',
    'case_of_browsers',
    'SeleniumAssertion',
)



# -*- coding: utf-8 -*-

import sys
from threading import Lock
from contextlib import contextmanager

from . import xunit
from . import reason
from . import runnable
from .utils import pyv
from .utils import colors
from .utils.common import MPSupportedValue


DEFAULT_NAME = 'seismograph'


lock = Lock()


def get_runnable_from_storage_item(item):
    runnable_object, _ = item
    return runnable_object


def get_xunit_data_from_storage_item(item):
    _, xunit_data = item
    return xunit_data


def get_last_item_from_storage(storage):
    if storage:
        return storage[len(storage) - 1]
    return None


def get_xunit_data_from_storage(storage, runnable_object):
    for item in storage:
        if get_runnable_from_storage_item(item) == runnable_object:
            return get_xunit_data_from_storage_item(item)
    return None


def reset_item_of_storage(storage, runnable_object, xunit_data):
    assert isinstance(xunit_data, xunit.XUnitData)

    for item in storage:
        if get_runnable_from_storage_item(item) == runnable_object:
            storage.remove(item)
            storage.append((runnable_object, xunit_data))
            return True

    return False


def get_runtime_from_storage(storage):
    rt = float()
    for item in storage:
        rt += get_xunit_data_from_storage_item(item).runtime
    return rt


class ResultConsole(object):

    class ChildConsole(object):

        TAB = 2  # tab to spaces
        DEFAULT_TABS = 1

        def __init__(self):
            self.__buffer = []
            self.__tabs = self.DEFAULT_TABS * self.TAB

        def __call__(self, *strings):
            string = []

            for s in strings:
                if not isinstance(s, pyv.basestring):
                    s = pyv.unicode_string(s)
                string.append(s)

            self.__buffer.append(
                ''.join(' ' for _ in pyv.xrange(self.__tabs)) + u''.join(string) + '\n',
            )

        @property
        def set_trace(self):
            try:
                import ipdb as pdb
            except ImportError:
                import pdb
            return pdb.set_trace

        @staticmethod
        def prompt(prompt=None):
            return raw_input(prompt)

        @contextmanager
        def ul(self, title=None, numerate=False):
            li = []

            yield lambda s: li.append(s)

            if title:
                self(u'{}:'.format(title))
            for l in li:
                self(
                    u'{}{} {}'.format(
                        ''.join(' ' for _ in pyv.xrange(self.__tabs)),
                        '{}.'.format(li.index(l) + 1) if numerate else '*', l,
                    ),
                )

        @contextmanager
        def tab(self):
            current_tabs = self.__tabs
            self.__tabs = current_tabs + self.TAB
            yield
            self.__tabs = current_tabs

        def flush(self, stdout):
            stdout.write(
                u''.join(self.__buffer),
            )
            self.__buffer = []

    def __init__(self, stdout=None, verbose=False):
        self.__buffer = []
        self.__children = []

        self.__verbose = verbose
        self.__stdout = stdout or sys.stdout

    def __call__(self, string):
        self.writeln(string)
        self.flush()

    def child_console(self):
        child_console = self.ChildConsole()
        self.__children.append(child_console)
        return child_console

    def flush(self):
        with lock:
            self.__stdout.write(
                u''.join(self.__buffer),
            )
            for child in self.__children:
                child.flush(self.__stdout)

        self.__buffer = []

    def write(self, string):
        self.__buffer.append(string)

    def writeln(self, string):
        self.write(string + '\n')

    def line_break(self):
        self.writeln('')


class ResultMarkers(object):

        LONG_FAIL = 'FAIL'
        LONG_SKIP = 'SKIP: '
        LONG_SUCCESS = 'OK'
        LONG_ERROR = 'ERROR'

        SMALL_FAIL = 'F'
        SMALL_SKIP = 'S'
        SMALL_ERROR = 'E'
        SMALL_SUCCESS = '.'

        def __init__(self, config):
            self.__config = config

        @property
        def config(self):
            return self.__config

        def fail(self):
            if self.__config.VERBOSE:
                if self.__config.NO_COLOR:
                    return self.LONG_FAIL
                return colors.yellow(self.LONG_FAIL)
            if self.__config.NO_COLOR:
                return self.SMALL_FAIL
            return colors.yellow(self.SMALL_FAIL)

        def skip(self, reason):
            if self.__config.VERBOSE:
                if self.__config.NO_COLOR:
                    return u'{}{}'.format(self.LONG_SKIP, reason)
                return u'{}{}'.format(colors.blue(self.LONG_SKIP), reason)
            if self.__config.NO_COLOR:
                return self.SMALL_SKIP
            return colors.blue(self.SMALL_SKIP)

        def success(self):
            if self.__config.VERBOSE:
                if self.__config.NO_COLOR:
                    return self.LONG_SUCCESS
                return colors.green(self.LONG_SUCCESS)
            return self.SMALL_SUCCESS

        def error(self):
            if self.__config.VERBOSE:
                if self.__config.NO_COLOR:
                    return self.LONG_ERROR
                return colors.red(self.LONG_ERROR)
            if self.__config.NO_COLOR:
                return self.SMALL_ERROR
            return colors.red(self.SMALL_ERROR)


class State(object):

    def __init__(self, result, should_stop=False):
        self.__result = result
        self.__should_stop = MPSupportedValue(should_stop)

    def support_mp(self, should_stop=None):
        if should_stop:
            self.__should_stop.set_mp(should_stop)

    @property
    def should_stop(self):
        return self.__should_stop.value

    @should_stop.setter
    def should_stop(self, value):
        self.__should_stop.value = value

    @property
    def runtime(self):
        runtime = float()

        for storage in (
                self.__result.errors,
                self.__result.skipped,
                self.__result.failures,
                self.__result.successes):
            runtime += get_runtime_from_storage(storage)

        return round(runtime, xunit.ROUND_RUNTIME)

    @property
    def tests(self):
        return self.errors + self.successes + self.failures + self.skipped

    @property
    def errors(self):
        return len(self.__result.errors)

    @property
    def successes(self):
        return len(self.__result.successes)

    @property
    def failures(self):
        return len(self.__result.failures)

    @property
    def skipped(self):
        return len(self.__result.skipped)

    @property
    def was_success(self):
        return not self.__result.errors and not self.__result.failures


class Result(object):

    __marker_class__ = ResultMarkers

    def __init__(self, config, name=None, stdout=None, current_state=None, is_proxy=False):
        self.errors = []
        self.skipped = []
        self.failures = []
        self.successes = []

        self.proxies = []

        self.__config = config
        self.__is_proxy = is_proxy
        self.__name = name or DEFAULT_NAME
        self.__current_state = current_state or State(self)

        self._stdout = stdout or sys.stdout
        self._marker = self.__marker_class__(self.__config)

        self.__console = ResultConsole(
            self._stdout,
            verbose=self.__config.VERBOSE,
        )

        if not is_proxy:
            global lock

            if self.__config.GEVENT:
                pyv.check_gevent_supported()

                from gevent.lock import Semaphore

                lock = Semaphore()

            if self.__config.MULTIPROCESSING:
                from multiprocessing import Lock

                lock = Lock()

    def __enter__(self):
        self.print_begin()
        return self

    def __exit__(self, *args, **kwargs):
        self.print_final()

        if self.__config.XUNIT_REPORT:
            self.create_report(self.__config.XUNIT_REPORT)

    def __repr__(self):
        state = self.get_state()
        return '<Result(tests={}, failures={}, errors={}, skipped={} success={})>'.format(
            state.tests,
            state.failures,
            state.errors,
            state.skipped,
            state.successes,
        )

    @property
    def name(self):
        return self.__name

    @property
    def is_proxy(self):
        return self.__is_proxy

    @property
    def config(self):
        return self.__config

    @property
    def console(self):
        return self.__console

    @property
    def current_state(self):
        return self.__current_state

    def create_proxy(self, **kwargs):
        return self.__class__(
            self.__config,
            is_proxy=True,
            stdout=self._stdout,
            current_state=self.__current_state,
            **kwargs
        )

    def extend(self, result):
        assert result.is_proxy, 'result can not be extended from no proxy'

        self.errors.extend(result.errors)
        self.skipped.extend(result.skipped)
        self.failures.extend(result.failures)
        self.successes.extend(result.successes)

    @contextmanager
    def proxy(self, runnable_object=None):
        proxy = self.create_proxy(
            name=runnable.class_name(runnable_object) if runnable_object else None,
        )
        if runnable_object:
            self.proxies.append(proxy)

        try:
            yield proxy
        finally:
            self.extend(proxy)
            proxy.console.flush()

    def get_state(self):
        return State(
            self, should_stop=self.__current_state.should_stop,
        )

    def get_fail_by(self, runnable_object):
        return get_xunit_data_from_storage(self.failures, runnable_object)

    def reset_fail(self, runnable_object, xunit_data):
        return reset_item_of_storage(self.failures, runnable_object, xunit_data)

    def get_error_by(self, runnable_object):
        return get_xunit_data_from_storage(self.errors, runnable_object)

    def reset_error(self, runnable_object, xunit_data):
        return reset_item_of_storage(self.errors, runnable_object, xunit_data)

    def get_skip_by(self, runnable_object):
        return get_xunit_data_from_storage(self.skipped, runnable_object)

    def reset_skip(self, runnable_object, xunit_data):
        return reset_item_of_storage(self.skipped, runnable_object, xunit_data)

    def get_success_by(self, runnable_object):
        return get_xunit_data_from_storage(self.successes, runnable_object)

    def reset_success(self, runnable_object, xunit_data):
        return reset_item_of_storage(self.successes, runnable_object, xunit_data)

    def add_error(self, runnable_object, traceback, runtime, exc):
        error_reason = reason.create(
            runnable_object, traceback, config=self.__config,
        )

        xunit_data = xunit.XUnitData(
            exc=exc,
            runtime=runtime,
            reason=reason.format_reason(error_reason),
            class_name=runnable.class_name(runnable_object),
            method_name=runnable.stopped_on(runnable_object),
        )

        self.errors.append((runnable_object, xunit_data))
        self.print_finish(self._marker.error())

        if self.__config.STOP:
            self.__current_state.should_stop = True

    def add_fail(self, runnable_object, traceback, runtime, exc):
        fail_reason = reason.create(
            runnable_object, traceback, config=self.__config,
        )

        xunit_data = xunit.XUnitData(
            exc=exc,
            runtime=runtime,
            reason=reason.format_reason(fail_reason),
            class_name=runnable.class_name(runnable_object),
            method_name=runnable.stopped_on(runnable_object),
        )

        self.failures.append((runnable_object, xunit_data))
        self.print_finish(self._marker.fail())

        if self.__config.STOP:
            self.__current_state.should_stop = True

    def add_success(self, runnable_object, runtime):
        xunit_data = xunit.XUnitData(
            runtime=runtime,
            class_name=runnable.class_name(runnable_object),
            method_name=runnable.method_name(runnable_object),
        )

        self.successes.append((runnable_object, xunit_data))
        self.print_finish(self._marker.success())

    def add_skip(self, runnable_object, reason, runtime):
        xunit_data = xunit.XUnitData(
            reason=reason,
            runtime=runtime,
            class_name=runnable.class_name(runnable_object),
            method_name=runnable.method_name(runnable_object),
        )

        self.skipped.append((runnable_object, xunit_data))
        self.print_finish(self._marker.skip(reason))

    def create_report(self, file_path):
        with open(file_path, 'w') as f:
            f.write(
                xunit.create_xml_document(self),
            )

    def print_start(self, runnable_object):
        if self.__config.VERBOSE:
            self.__console.write(
                '* {}: '.format(str(runnable_object)),
            )

    def print_finish(self, status):
        if self.__config.VERBOSE:
            self.__console.writeln(status)
        else:
            self.__console.write(status)

    def print_begin(self):
        self.__console.writeln('Seismograph is measuring:')
        self.__console.line_break()
        self.console.flush()

    def print_final(self):
        if not self.__config.VERBOSE:
            self.__console.line_break()

        need_report = bool(self.errors or self.failures)

        if need_report:
            self.__console.line_break()
            self.__console.writeln(
                'Seismograph is reporting about reasons:',
            )
            self.__console.line_break()

            for storage in (self.errors, self.failures):
                for storage_item in storage:
                    runnable_object, xunit_data = storage_item
                    if xunit_data.reason:
                        crash_reason = reason.create(
                            runnable_object, xunit_data.reason, config=self.__config,
                        )
                        self.__console.writeln(
                            reason.format_reason_to_output(crash_reason),
                        )

        total = 'tests={} failures={} errors={} skipped={} successes={} runtime={}'.format(
            self.__current_state.tests,
            self.__current_state.failures,
            self.__current_state.errors,
            self.__current_state.skipped,
            self.__current_state.successes,
            self.__current_state.runtime,
        )

        sep_line = ''.join(
            '-' for _ in pyv.xrange(len(total)),
        )

        if not need_report:
            self.__console.line_break()

        self.__console.writeln(sep_line)
        self.__console.writeln(total)
        self.__console.flush()



# -*- coding: utf-8 -*-

from copy import deepcopy

from .exceptions import ExtensionNotFound


_TMP = {}


def install(ext, program):
    assert isinstance(ext, BaseExtension), \
        '"{}" is not instance of "BaseExtension"'.format(
            ext.__class__.__name__,
        )
    ext.__install__(program)


def add_options(ext, parser):
    ext.__add_options__(parser)


class BaseExtension(object):

    def __call__(self, *args, **kwargs):
        raise NotImplementedError(
            'Method "__call__" not implemented in "{}.{}"'.format(
                self.__class__.__module__, self.__class__.__name__,
            ),
        )

    def __install__(self, program):
        raise NotImplementedError(
            'Method "__install__" not implemented in "{}.{}"'.format(
                self.__class__.__module__, self.__class__.__name__,
            ),
        )

    @staticmethod
    def __add_options__(parser):
        pass


class ExtensionContainer(object):

    def __init__(self, ext, args=None, kwargs=None):
        self.__ext = ext
        self.__args = args or tuple()
        self.__kwargs = kwargs or dict()

    def __call__(self):
        return self.__ext(*self.__args, **self.__kwargs)

    @property
    def ext(self):
        return self.__ext

    @property
    def args(self):
        return self.__args

    @property
    def kwargs(self):
        return self.__kwargs


class SingletonExtensionContainer(ExtensionContainer):

    def __call__(self):
        return self.ext


def get(name):
    try:
        ext = _TMP[name]
    except KeyError:
        raise ExtensionNotFound(name)

    if isinstance(ext, ExtensionContainer):
        return ext()

    return deepcopy(ext)


def set(ext, name, is_data=False, singleton=False, args=None, kwargs=None):
    if is_data:
        _TMP[name] = ext
    else:
        if singleton:
            _TMP[name] = SingletonExtensionContainer(
                ext(*(args or tuple()), **(kwargs or dict())),
            )
        else:
            _TMP[name] = ExtensionContainer(ext, args=args, kwargs=kwargs)


def clear():
    _TMP.clear()



# -*- coding: utf8 -*-

from __future__ import absolute_import

import time
from random import randint
from functools import wraps

try:
    from httplib import HTTPException
except ImportError:  # please python 3
    from http.client import HTTPException

from selenium.common.exceptions import WebDriverException
from selenium.webdriver.common.action_chains import ActionChains as __ActionChains

from ...utils import pyv
from ...exceptions import TimeoutException
from .exceptions import ReRaiseWebDriverException
from .utils import change_name_from_python_to_html


DEFAULT_POLLING_TIMEOUT = 30
POLLING_EXCEPTIONS = (
    IOError,
    OSError,
    HTTPException,
    WebDriverException,
)


class ActionChains(__ActionChains):

    def __init__(self, proxy):
        super(ActionChains, self).__init__(proxy.driver)

    def __call__(self, proxy):
        return self.__class__(proxy)

    def reset(self):
        self._actions = []

    def perform(self, reset=True):
        super(ActionChains, self).perform()

        if reset:
            self.reset()


def create_screen_file_name():
    file_ex = '.png'
    file_name = str(
        int(time.time() + randint(0, 1000)),
    )
    file_name += file_ex
    return file_name


class WebElementToObject(object):

    def __init__(self, proxy, allow_raise=True):
        self.__dict__['__proxy__'] = proxy
        self.__dict__['__allow_raise__'] = allow_raise

    @property
    def css(self):
        return WebElementCssToObject(
            self.__dict__['__proxy__'],
        )

    def __getattr__(self, item):
        atr = self.__dict__['__proxy__'].get_attribute(
            change_name_from_python_to_html(item),
        )

        if atr:
            return atr

        if self.__dict__['__allow_raise__']:
            raise AttributeError(item)

    def __setattr__(self, key, value):
        self.__dict__['__proxy__'].parent.execute_script(
            'arguments[0].setAttribute(arguments[1], arguments[2]);',
            self.__dict__['__proxy__'],
            change_name_from_python_to_html(key),
            value
        )


class WebElementCssToObject(object):

    def __init__(self, proxy):
        self.__dict__['__proxy__'] = proxy

    def __getattr__(self, item):
        return self.__dict__['__proxy__'].value_of_css_property(
            change_name_from_python_to_html(item),
        )

    def __setattr__(self, key, value):
        self.__dict__['__proxy__'].parent.execute_script(
            'arguments[0].style[arguments[1]] = arguments[2];',
            self.__dict__['__proxy__'],
            change_name_from_python_to_html(key),
            value
        )


def make_object(web_element, allow_raise=True):
    return WebElementToObject(web_element, allow_raise=allow_raise)


def polling(callback=None, timeout=DEFAULT_POLLING_TIMEOUT):
    def wrapper(f):
        @wraps(f)
        def wrapped(*args, **kwargs):
            exc = None
            t_start = time.time()

            while time.time() <= t_start + timeout:
                try:
                    return f(*args, **kwargs)
                except POLLING_EXCEPTIONS as error:
                    exc = error
                    continue
            else:
                if exc:
                    raise exc
                raise

        return wrapped

    if callback is not None:
        return wrapper(callback)

    return wrapper


def re_raise_wd_exc(callback=None, exc_cls=ReRaiseWebDriverException, message=None):
    def wrapper(f):
        @wraps(f)
        def wrapped(*args, **kwargs):
            try:
                return f(*args, **kwargs)
            except (TimeoutException, WebDriverException) as e:
                raise exc_cls(message or pyv.get_exc_message(e))

        return wrapped

    if callable(callback):
        return wrapper(callback)

    return wrapper



# -*- coding: utf-8 -*-

from optparse import OptionGroup

from .mock import BaseMock
from .base import BaseMockServer
from .json_api_mock import JsonMock
from .json_api_mock import JsonApiMockServer

from ...extensions import BaseExtension
from ...exceptions import EmergencyStop


SERVER_TYPES = {
    'simple': BaseMockServer,
    'json_api': JsonApiMockServer,
}


class MockServerExtension(BaseExtension):

    __ex_name__ = 'mock_server'
    __config_key__ = 'MOCK_SERVER_EX'
    __default_server_type__ = 'json_api'

    @staticmethod
    def __add_options__(parser):
        group = OptionGroup(parser, 'MockServer extension options')

        group.add_option(
            '--mock-server-mocks-dir',
            dest='MOCK_SERVER_MOCKS_DIR',
            default=None,
        )
        group.add_option(
            '--mock-server-host',
            dest='MOCK_SERVER_HOST',
            default=None,
        )
        group.add_option(
            '--mock-server-port',
            dest='MOCK_SERVER_PORT',
            default=None,
            type=int,
        )
        group.add_option(
            '--mock-server-debug',
            dest='MOCK_SERVER_DEBUG',
            action='store_true',
            default=False,
        )

        parser.add_option_group(group)

    def __install__(self, program):
        params = program.config.get(self.__config_key__, {})

        ex_kwargs = {
            'mocks': params.get('MOCKS'),
            'host': program.config.MOCK_SERVER_HOST or params.get('HOST'),
            'port': program.config.MOCK_SERVER_PORT or params.get('PORT'),
            'debug': program.config.MOCK_SERVER_DEBUG or params.get('DEBUG'),
            'mocks_path': program.config.MOCK_SERVER_MOCKS_DIR or params.get('MOCKS_PATH'),
            'server_type': params.get(
                'SERVER_TYPE', self.__default_server_type__,
            ),
        }

        program.shared_extension(
            self.__ex_name__, self, singleton=True, kwargs=ex_kwargs,
        )

    def __call__(self,
                 host=None,
                 port=None,
                 mocks=None,
                 debug=False,
                 mocks_path=None,
                 server_type=__default_server_type__):
        try:
            mock_server_class = SERVER_TYPES[server_type]
        except KeyError:
            raise EmergencyStop(
                'Incorrect server type "{}"'.format(server_type),
            )

        return mock_server_class(
            mocks_path,
            host=host,
            port=port,
            mocks=mocks,
            debug=debug,
        )


__all__ = (
    'BaseMock',
    'JsonMock',
    'SERVER_TYPES',
    'BaseMockServer',
    'JsonApiMockServer',
)



# -*- coding: utf-8 -*-

"""
Default groups for simple run
"""

from .. import runnable


class DefaultSuiteGroup(runnable.RunnableGroup):

    def run(self, result):
        self._is_run = True

        for suite in self.objects:
            suite(result)


class DefaultCaseGroup(runnable.RunnableGroup):

    def run(self, result):
        self._is_run = True

        for case in self.objects:
            case(result)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from types import MethodType
from contextlib import contextmanager

from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement
from selenium.common.exceptions import NoSuchElementException

from .router import Router
from .tools import polling
from .tools import make_object
from .tools import ActionChains
from .query import QueryProcessor


METHOD_ALIASES = {
    'go_to': 'get',
    'set': 'send_keys',
}


def factory_method(f, driver, config, allow_polling=True, reason_storage=None):
    def wrapper(*args, **kwargs):
        result = f(*args, **kwargs)

        if isinstance(result, WebElement):
            return WebElementProxy(
                result,
                config=config,
                driver=driver,
                allow_polling=allow_polling,
                reason_storage=reason_storage,
            )

        if isinstance(result, list):
            we_list = WebElementList()

            for obj in result:
                if isinstance(obj, WebElement):
                    we_list.append(
                        WebElementProxy(
                            obj,
                            config=config,
                            allow_polling=allow_polling,
                            reason_storage=reason_storage,
                        ),
                    )
                else:
                    return result

            return we_list
        return result
    return wrapper


class WebElementList(list):

    def get_by(self, **kwargs):
        for we in self:
            if all(getattr(we.attr, k) == v for k, v in kwargs.items()):
                return we
        else:
            raise NoSuchElementException(
                u'by filters "{}" of list {}'.format(
                    u', '.join(
                        u'{}={}'.format(k, v) for k, v in kwargs.items()
                    ),
                    repr(self),
                ),
            )

    def filter(self, **kwargs):
        result = []

        for we in self:
            if all(getattr(we.attr, k) == v for k, v in kwargs.items()):
                result.append(we)

        return result


class BaseProxy(object):

    def __init__(self, wrapped, config=None, driver=None, reason_storage=None, allow_polling=True):
        self.__dict__['__driver__'] = driver
        self.__dict__['__config__'] = config
        self.__dict__['__wrapped__'] = wrapped
        self.__dict__['__query__'] = QueryProcessor(self)
        self.__dict__['__allow_polling__'] = allow_polling
        self.__dict__['__reason_storage__'] = reason_storage or dict()

        assert self.__class__ != BaseProxy, 'This is base class only. Can not be instanced.'

    def __repr__(self):
        return repr(self._wrapped)

    def __getattr__(self, item):
        attr = getattr(
            self._wrapped, METHOD_ALIASES.get(item, item), None,
        )

        if attr is None:
            attr = getattr(self.query, item)

        if callable(attr) and type(attr) == MethodType:
            if self.allow_polling:
                return polling(
                    callback=factory_method(
                        attr,
                        self.driver,
                        self.config,
                        allow_polling=self.allow_polling,
                        reason_storage=self.reason_storage,
                    ),
                    timeout=self.config.POLLING_TIMEOUT,
                )

            return factory_method(
                attr,
                self.driver,
                self.config,
                allow_polling=self.allow_polling,
                reason_storage=self.reason_storage,
            )

        return attr

    @property
    def _wrapped(self):
        return self.__dict__['__wrapped__']

    @property
    def config(self):
        return self.__dict__['__config__']

    @property
    def query(self):
        return self.__dict__['__query__']

    @property
    def driver(self):
        return self.__dict__['__driver__']

    @property
    def allow_polling(self):
        return (
            bool(self.config.POLLING_TIMEOUT) and self.__dict__['__allow_polling__']
        )

    @property
    def reason_storage(self):
        return self.__dict__['__reason_storage__']

    @property
    def text(self):
        def get_text():
            if isinstance(self._wrapped, WebElement):
                return self._wrapped.text
            return self._wrapped.find_element_by_tag_name('body').text

        if self.allow_polling:
            func = polling(
                get_text,
                timeout=self.config.POLLING_TIMEOUT,
            )
            return func()

        return get_text()

    @contextmanager
    def disable_polling(self):
        try:
            self.__dict__['__allow_polling__'] = False
            yield
        finally:
            self.__dict__['__allow_polling__'] = True


class WebElementProxy(BaseProxy):

    def __init__(self, *args, **kwargs):
        super(WebElementProxy, self).__init__(*args, **kwargs)

        assert isinstance(self._wrapped, WebElement), 'This is proxy to WebElement only'

    @property
    def attr(self):
        return make_object(self, allow_raise=False)


class WebDriverProxy(BaseProxy):

    def __init__(self, *args, **kwargs):
        super(WebDriverProxy, self).__init__(*args, **kwargs)

        assert isinstance(self._wrapped, WebDriver), 'This is proxy to WebDriver only'

        self.__dict__['__router__'] = Router(self)
        self.__dict__['__action_chains__'] = ActionChains(self)

    @property
    def driver(self):
        return self

    @property
    def router(self):
        return self.__dict__['__router__']

    @property
    def action_chains(self):
        return self.__dict__['__action_chains__']

    @property
    def current_url(self):
        if self.allow_polling:
            func = polling(
                lambda: self._wrapped.current_url,
                timeout=self.config.POLLING_TIMEOUT,
            )
            return func()

        return self._wrapped.current_url



# -*- coding: utf-8 -*-

from functools import wraps

from selenium.common.exceptions import NoSuchElementException

from ..query import QueryObject
from ..exceptions import FieldError


def selector(**kwargs):
    """
    proxy for tag attributes
    """
    return kwargs


def fill_field_handler(f):
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        result = f(self, *args, **kwargs)

        self.group.fill_memo.add(self)

        return result
    return wrapper


def clear_field_handler(f):
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        result = f(self, *args, **kwargs)

        try:
            self.group.fill_memo.remove(self)
        except KeyError:
            pass

        return result
    return wrapper


class SimpleFieldInterface(object):

    @property
    def weight(self):
        raise NotImplementedError(
            'Property "weight" not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )

    def fill(self, value=None):
        raise NotImplementedError(
            'Method "fill" not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )

    def clear(self):
        raise NotImplementedError(
            'Method "clear" not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )


class FormField(object):
    """
    Base class for all fields
    """

    __tag__ = None

    def __init__(self,
                 name,
                 value=None,
                 required=False,
                 selector=None,
                 error_mess=None,
                 invalid_value=None,
                 weight=None):

        self.name = name

        if not isinstance(selector, dict):
            raise ValueError('incorrect selector')

        self.value = value
        self.required = required
        self.error_mess = error_mess
        self.invalid_value = invalid_value

        self.__group = None

        self.__weight = weight
        self.__selector = selector

    @property
    def query(self):
        return self.we.query

    def bind(self, group):
        self.__group = group

        if callable(self.value):
            self.value = self.value()

        if callable(self.invalid_value):
            self.invalid_value = self.invalid_value()

    @property
    def driver(self):
        return self.group.driver

    @property
    def group(self):
        if not self.__group:
            raise RuntimeError(
                'Field is not binding to group',
            )
        return self.__group

    @property
    def weight(self):
        return self.__weight

    @property
    def selector(self):
        return self.__selector

    @property
    def we(self):
        return self.group.query.from_object(
            QueryObject(self.__tag__, **self.__selector),
        ).first()

    @property
    def attr(self):
        return self.we.attr


class Input(FormField, SimpleFieldInterface):

    __tag__ = 'input'

    @fill_field_handler
    def fill(self, value=None):
        value = value or self.value

        self.we.send_keys(*value)

    @clear_field_handler
    def clear(self):
        self.we.clear()


assert issubclass(Input, SimpleFieldInterface)


class TextArea(Input):

    __tag__ = 'textarea'


assert issubclass(TextArea, SimpleFieldInterface)


class Checkbox(FormField, SimpleFieldInterface):

    __tag__ = 'input'

    @fill_field_handler
    def fill(self, value=None):
        value = value or self.value

        el = self.we
        current_value = el.is_selected()

        if self.group.__allow_raises__:

            if current_value and value:
                raise FieldError('Oops, checkbox was selected')
            elif not current_value and not value:
                raise FieldError('Oops, checkbox was unselected')

        if (value and not current_value) or (not value and current_value):
            el.click()

    @clear_field_handler
    def clear(self):
        el = self.we

        if el.is_selected():
            el.click()


assert issubclass(Checkbox, SimpleFieldInterface)


class RadioButton(Checkbox):

    __tag__ = 'input'

    @fill_field_handler
    def fill(self, value=None):
        value = value or self.value

        if value is None:
            return False

        el = self.we
        current_value = el.is_selected()
        changed = False

        if (value and not current_value) or (not value and current_value):
            el.click()
            changed = True

        return changed

    @clear_field_handler
    def clear(self):
        pass


assert issubclass(RadioButton, SimpleFieldInterface)


class Select(FormField, SimpleFieldInterface):

    __tag__ = 'select'

    @fill_field_handler
    def fill(self, value=None):
        value = value or self.value

        options = filter(
            lambda opt: opt.get_attribute('value') == value,
            self.we.find_elements_by_tag_name('option'),
        )

        for option in options:
            option.click()
        else:
            raise NoSuchElementException(
                u'Can not select value "{}" of field "{}", selector "{}"'.format(
                    value, self.name, str(self.selector),
                ),
            )

    @clear_field_handler
    def clear(self):
        pass


assert issubclass(Select, SimpleFieldInterface)



# -*- coding: utf-8 -*-


def create_reason(name, desc, *args):
    return u'{} ({}): \n{}\n\n'.format(
        name, desc, u'\n'.join(u'  {}'.format(s) for s in args),
    )


def get_pool_size_of_value(value, in_two=False):
    from multiprocessing import cpu_count

    size = value

    if size <= 0:
        size = cpu_count()

    if in_two:
        return int(round(size / 2)) or 2

    return size



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from selenium.webdriver.ie.webdriver import WebDriver as IeWebDriver
from selenium.webdriver.opera.webdriver import WebDriver as OperaWebDriver
from selenium.webdriver.remote.webdriver import WebDriver as RemoteWebDriver
from selenium.webdriver.chrome.webdriver import WebDriver as ChromeWebDriver
from selenium.webdriver.firefox.webdriver import WebDriver as FirefoxWebDriver
from selenium.webdriver.phantomjs.webdriver import WebDriver as PhantomJSWebDriver


IE = 'ie'
OPERA = 'opera'
CHROME = 'chrome'
FIREFOX = 'firefox'
PHANTOMJS = 'phantomjs'


NAME_TO_DRIVER = {
    IE: IeWebDriver,
    OPERA: OperaWebDriver,
    CHROME: ChromeWebDriver,
    FIREFOX: FirefoxWebDriver,
    PHANTOMJS: PhantomJSWebDriver,
}


__all__ = (
    IE,
    OPERA,
    CHROME,
    FIREFOX,
    PHANTOMJS,
    IeWebDriver,
    NAME_TO_DRIVER,
    OperaWebDriver,
    RemoteWebDriver,
    ChromeWebDriver,
    FirefoxWebDriver,
    PhantomJSWebDriver,
)



# -*- coding: utf-8 -*-

import sys
import time
from contextlib import contextmanager

from ..exceptions import TimeoutException


WAITING_FOR_TIMEOUT = 30


def waiting_for(func, timeout=None, sleep=None, args=None, kwargs=None):
    args = args or tuple()
    kwargs = kwargs or {}

    timeout = timeout or WAITING_FOR_TIMEOUT

    t_start = time.time()

    while time.time() <= t_start + timeout:
        result = func(*args, **kwargs)

        if result:
            return result

        if sleep:
            time.sleep(sleep)
    else:
        raise TimeoutException(
            'Timeout {} exceeded'.format(timeout),
        )


def call_to_chain(chain, method_name, *args, **kwargs):
    for obj in chain:
        if method_name:
            getattr(obj, method_name)(*args, **kwargs)
        else:
            obj(*args, **kwargs)


def measure_time():
    start_time = time.time()
    return lambda: time.time() - start_time


@contextmanager
def dev_null():
    class MockStd(object):

        def __getattr__(self, item):
            def null(*args, **kwargs):
                pass
            return null

    stdout = sys.stdout
    sys.stdout = MockStd()
    try:
        yield
    finally:
        sys.stdout = stdout


class MPSupportedValue(object):

    def __init__(self, value=None):
        self._value = value

    @property
    def value(self):
        if hasattr(self._value, 'value'):
            return self._value.value
        return self._value

    @value.setter
    def value(self, value):
        if hasattr(self._value, 'value'):
            self._value.value = value
        else:
            self._value = value

    def set_mp(self, value):
        self._value = value



# -*- coding: utf-8 -*-

from ...exceptions import SeismographError


class SeleniumExError(SeismographError):
    pass


class RouterError(SeleniumExError):
    pass


class RouteNotFound(RouterError):
    pass


class ReRaiseWebDriverException(SeleniumExError):
    pass


class FieldError(SeleniumExError):
    pass



# coding: utf-8


class DictObject(dict):

    def __getattr__(self, item):
        try:
            return self[item]
        except KeyError:
            raise AttributeError(
                '"{}" does not have "{}" attribute.'.format(self.__class__.__name__, item),
            )

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, item):
        try:
            del self[item]
        except KeyError:
            raise AttributeError(
                '"{}" does not have "{}" attribute.'.format(self.__class__.__name__, item),
            )

    def copy(self):
        new_dct = self.__class__()
        new_dct.update(
            super(DictObject, self).copy(),
        )
        return new_dct


class Context(DictObject):
    pass



# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages


__version__ = '0.0.2'


if __name__ == '__main__':
    setup(
        name='seismograph',
        version=__version__,
        url='https://github.com/trifonovmixail/seismograph',
        packages=find_packages(exclude=('example*', 'tests*')),
        author='Mikhail Trifonov',
        author_email='trifonovmixail@ya.ru',
        license='GNU LGPL',
        description='Framework for test development',
        keywords='test unittest framework',
        long_description=open('README.rst').read(),
        include_package_data=True,
        zip_safe=False,
        platforms='any',
        install_requires=(
            'six',
            'requests',
            'flask>=0.7',
            'selenium==2.46.0',
        ),
        entry_points={
            'console_scripts': (
                'seismograph = seismograph.__main__:main',
                'seismograph.mock_server = seismograph.ext.mock_server.__main__:main',
            ),
        },
        test_suite='tests',
        classifiers=(
            'Development Status :: 3 - Alpha',
            'Framework :: Seismograph',
            'Natural Language :: Russian',
            'Intended Audience :: Developers',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: Implementation :: CPython',
            'Topic :: Software Development :: Testing',
        ),
    )



# -*- coding: utf-8 -*-

from copy import deepcopy
from contextlib import contextmanager

from six import with_metaclass

from .fields import FormField
from ..query import QueryObject
from .iterators import FieldsIterator
from .fields import SimpleFieldInterface
from .iterators import RequiredFieldsIterator
from .iterators import FieldsWithContainsInvalidValueIterator


def make_field(group_class, weight=None, name=None):
    if not issubclass(group_class, FieldsGroup):
        raise ValueError('group_class is not FieldsGroup subclass')

    return GroupContainer(group_class, weight, name)


@contextmanager
def preserve_original(group, ignore_exc=None):
    try:
        if ignore_exc:
            try:
                yield
            except ignore_exc:
                pass
        else:
            yield
    finally:
        group.reload()


def iter_fields(group, exclude=None):
    assert isinstance(group, FieldsGroup)
    return FieldsIterator(group, exclude=exclude)


def iter_required(group, exclude=None):
    assert isinstance(group, FieldsGroup)
    return RequiredFieldsIterator(group, exclude=exclude)


def iter_invalid(group, exclude=None):
    assert isinstance(group, FieldsGroup)
    return FieldsWithContainsInvalidValueIterator(group, exclude=exclude)


class GroupContainer(object):

    def __init__(self, group_class, weight, name):
        self.__group_class = group_class
        self.__weight = weight
        self.__name = name

    @property
    def name(self):
        return self.__name

    def __call__(self, group):
        return self.__group_class(
            group.driver,
            parent=group,
            name=self.__name,
            weight=self.__weight,
        )


class GroupMemento(dict):

    def _set_field(self, filed):
        self[filed] = {
            'value': filed.value,
            'required': filed.required,
            'error_mess': filed.error_mess,
            'invalid_value': filed.invalid_value,
        }

    def get_field(self, filed):
        return self.get(filed)

    def add_field(self, field):
        assert isinstance(field, FormField)
        self._set_field(field)

    def restore(self, field_list):
        for field in field_list:
            orig = self.get_field(field)

            if orig:
                field.value = orig['value']
                field.required = orig['required']
                field.error_mess = orig['error_mess']
                field.invalid_value = orig['invalid_value']
            else:
                continue


class FieldsGroupMeta(type):

    def __call__(self, *args, **kwargs):
        instance = super(FieldsGroupMeta, self).__call__(*args, **kwargs)

        for atr in (a for a in dir(instance) if not a.startswith('_') and a != 'query'):
            maybe_field = getattr(instance, atr, None)

            if isinstance(maybe_field, (FormField, GroupContainer)):
                if maybe_field.name in instance.__exclude__:
                    try:
                        delattr(instance, atr)
                    except AttributeError:
                        pass
                    continue

                instance.add_field(atr, deepcopy(maybe_field))

        return instance


class FieldsGroup(with_metaclass(FieldsGroupMeta, SimpleFieldInterface)):

    __wrapper__ = None
    __remember__ = True
    __exclude__ = tuple()
    __allow_raises__ = True

    def __init__(self, proxy, weight=None, name=None, parent=None):
        if not hasattr(self, 'name'):
            self.name = name

        self.__fields = []
        self.__parent = parent
        self.__weight = weight
        self.__fill_memo = set()
        self.__memento = GroupMemento()
        self.__proxy = proxy

    @property
    def driver(self):
        return self.__proxy.driver

    @property
    def area(self):
        if self.__wrapper__:
            if not isinstance(self.__wrapper__, QueryObject):
                raise TypeError(
                    '"__wrapper__" can be instance of QueryObject only',
                )
            return self.__proxy.query.from_object(
                self.__wrapper__,
            ).first()

        return self.__proxy

    @property
    def query(self):
        return self.area.query

    @property
    def fields(self):
        return self.__fields

    @property
    def weight(self):
        return self.__weight

    @property
    def fill_memo(self):
        return self.__fill_memo

    def relate_to(self, proxy):
        self.__proxy = proxy

    def add_field(self, name, field):
        if isinstance(field, GroupContainer):
            field = field(self)
            setattr(self, name, field)
        elif isinstance(field, FormField):
            setattr(self, name, field)
            field.bind(self)
            self.__memento.add_field(field)
        else:
            raise TypeError('Unknown field type')

        self.__fields.append(field)
        self.__fields.sort(key=lambda f: f.weight)

    def add_subgroup(self, name, cls, weight=None):
        assert issubclass(cls, FieldsGroup),\
            '"{}" is not FieldsGroup subclass'.format(cls.__name__)

        self.add_field(
            name,
            make_field(
                cls, weight=weight, name=name,
            ),
        )

    def add_subform(self, *args, **kwargs):
        self.add_subgroup(*args, **kwargs)

    def reload(self):
        self.__memento.restore(self.__fields)

    def reset_memo(self):
        self.__fill_memo.clear()

        groups = (
            field for field in self.__fill_memo
            if isinstance(field, FieldsGroup)
        )

        for group in groups:
            group.reset_memo()

    def update(self, **kwargs):
        for field_name, value in kwargs.items():
            field = getattr(self, field_name, None)

            if isinstance(field, FieldsGroup):
                field.update(**value)
            elif isinstance(field, FormField):
                field.value = value
            else:
                raise LookupError(
                    'Field "{}" not found'.format(field_name),
                )

    def fill(self, exclude=None):
        exclude = exclude or tuple()

        for field in self.__fields:
            if (field not in self.__fill_memo) and (field not in exclude):
                field.fill()

        if self.__parent:
            self.__parent.fill_memo.add(self)

        self.reset_memo()

    def clear(self):
        for field in self.__fields:
            field.clear()

        self.reset_memo()


assert issubclass(FieldsGroup, SimpleFieldInterface)
